#include <math.h>
#include <string.h>
#include <stdio.h>

#include "gadaptivity.h"
#include "gtopology.h"
#include "vector.h"
#include "matrix.h"
#include "basefun.h"

/**
   Function interpolates values (stress or strain = derivatives) from nodes of element into selected point on element.
   
   @param bf - approximation functions on element
   @param rderfull - 1Darray of values in nodes; dimmension is ncomp*gt->nn after this manner {(val[0]; ... ;val[gt->nn])[0] ; ... ; (val[0]; ... ;val[gt->nn])[ncomp]}
   @param nodes - 1Darray of node numbers of element
   @param nn - total number of nodes on domain
   @param der_star - empty(returned) vector, dimension is ncomp
   
   created 19.8.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
      //   votoc "vector &der_star" za "long nn"
void give_der_star (vector &bf,double *rderfull,ivector &nodes,vector &der_star,long nn)
{
  long i,j,nne,ncomp;
  
  nne = nodes.n;
  ncomp = der_star.n;
  
  fillv (0.0,der_star);
  
  for (i=0;i<ncomp;i++)
    for (j=0;j<nne;j++)
      der_star[i] += bf[j] * rderfull[nodes[j] + i*nn];
  
}

void ntnmtov (matrix &ntnm,vector &ntnv)
{
  long i,j,nne;
  double mij,mii;

  nne = ntnv.n;

  mij = 0;
  for (i=0;i<nne;i++)
    for (j=0;j<nne;j++)
      mij += ntnm[i][j];

  mii = 0;
  for (i=0;i<nne;i++)
    mii += ntnm[i][i];

  for (i=0;i<nne;i++)
    ntnv[i] = ntnm[i][i] * mij / mii;
}


//  vytiskne do souboru typfile isolinie hodnot na prvku(valnod)
void print_contoures (gtopology *gt,char *file,double **midpoints,double *valel)
{
  long i,j,nmidp,nn,ne;
  long *nadjelnod,**adjelnod,**auxnod,*nodes;
  double **auxxyzv;
  vector valnod;
  
  nn = gt->nn;
  ne = gt->ne;
  
  auxxyzv = new double* [nn+ne];
  auxnod = new long* [ne];
  
  allocv (nn,valnod);
  
  // computing of valnod
  nadjelnod = gt->nadjelnod;
  adjelnod = gt->adjelnod;
  
  for (i=0;i<nn;i++){
    for (j=0;j<nadjelnod[i];j++){
      valnod[i] += valel[adjelnod[i][j]]; 
    }
    valnod[i] /= (double)nadjelnod[i]; 
  }
  
  nadjelnod = NULL;
  adjelnod = NULL;
  
  // assemble aux*
  for (i=0;i<nn;i++){
    auxxyzv[i] = new double [4];
    auxxyzv[i][0] = gt->gnodes[i].x;
    auxxyzv[i][1] = gt->gnodes[i].y;
    auxxyzv[i][2] = gt->gnodes[i].z;
    auxxyzv[i][3] = valnod[i];
  }
  for (i=0;i<ne;i++){
    if (midpoints[i] != NULL){
      auxxyzv[i+nn] = new double [4];
      auxxyzv[i+nn][0] = midpoints[i][0];
      auxxyzv[i+nn][1] = midpoints[i][1];
      auxxyzv[i+nn][2] = 0.0;
      auxxyzv[i+nn][3] = valel[i];
    }
    else
      auxxyzv[i+nn] = NULL;
  }
  
  nmidp = nn;
  for (i=0;i<ne;i++){
    nodes = gt->gelements[i].nodes;
    nmidp++;
    
    switch (gt->gelements[i].auxinf){
    case 312:{
      auxnod[i] = new long [4];
      auxnod[i][0] = 1;   auxnod[i][1] = nodes[0]+1;   auxnod[i][2] = nodes[1]+1;   auxnod[i][3] = nodes[2]+1;
      break;
    }
    case 412:{
      auxnod[i] = new long [13];
      auxnod[i][0] = 4;   auxnod[i][ 1] = nodes[0]+1;   auxnod[i][ 2] = nodes[1]+1;   auxnod[i][ 3] = nmidp;
                          auxnod[i][ 4] = nodes[1]+1;   auxnod[i][ 5] = nodes[2]+1;   auxnod[i][ 6] = nmidp;
                          auxnod[i][ 7] = nodes[2]+1;   auxnod[i][ 8] = nodes[3]+1;   auxnod[i][ 9] = nmidp;
                          auxnod[i][10] = nodes[3]+1;   auxnod[i][11] = nodes[0]+1;   auxnod[i][12] = nmidp;
      break;
    }
    case 622:{
      auxnod[i] = new long [13];
      auxnod[i][0] = 4;   auxnod[i][ 1] = nodes[0]+1;   auxnod[i][ 2] = nodes[3]+1;   auxnod[i][ 3] = nodes[5]+1;
                          auxnod[i][ 4] = nodes[3]+1;   auxnod[i][ 5] = nodes[1]+1;   auxnod[i][ 6] = nodes[4]+1;
                          auxnod[i][ 7] = nodes[4]+1;   auxnod[i][ 8] = nodes[2]+1;   auxnod[i][ 9] = nodes[5]+1;
                          auxnod[i][10] = nodes[3]+1;   auxnod[i][11] = nodes[4]+1;   auxnod[i][12] = nodes[5]+1;
      break;
    }
    case 822:{
      auxnod[i] = new long [25];
      auxnod[i][0] = 8;   auxnod[i][ 1] = nodes[0]+1;   auxnod[i][ 2] = nodes[4]+1;   auxnod[i][ 3] = nmidp;
                          auxnod[i][ 4] = nodes[4]+1;   auxnod[i][ 5] = nodes[1]+1;   auxnod[i][ 6] = nmidp;
                          auxnod[i][ 7] = nodes[1]+1;   auxnod[i][ 8] = nodes[5]+1;   auxnod[i][ 9] = nmidp;
                          auxnod[i][10] = nodes[5]+1;   auxnod[i][11] = nodes[2]+1;   auxnod[i][12] = nmidp;
                          auxnod[i][13] = nodes[2]+1;   auxnod[i][14] = nodes[6]+1;   auxnod[i][15] = nmidp;
                          auxnod[i][16] = nodes[6]+1;   auxnod[i][17] = nodes[3]+1;   auxnod[i][18] = nmidp;
                          auxnod[i][19] = nodes[3]+1;   auxnod[i][20] = nodes[7]+1;   auxnod[i][21] = nmidp;
                          auxnod[i][22] = nodes[7]+1;   auxnod[i][23] = nodes[0]+1;   auxnod[i][24] = nmidp;
      break;
    }
    case 413:{
      auxnod[i] = new long [5];
      auxnod[i][0] = 1;   auxnod[i][1] = nodes[0]+1;   auxnod[i][2] = nodes[1]+1;   auxnod[i][3] = nodes[2]+1;   auxnod[i][4] = nodes[3]+1;
      break;
    }
    default:{
      fprintf (stderr,"\n\n wrong dimdegnne in function");
      fprintf (stderr," print_contoures (%s, line %d)",__FILE__,__LINE__);
      break;
    }
    }
  }
  
  print_confile (file,nn,ne,auxxyzv,auxnod,gt->gelements[0].auxinf % 10);
  
  nodes = NULL;
  for (i=0;i<nn+ne;i++)
    delete [] auxxyzv[i];
  for (i=0;i<ne;i++)
    delete [] auxnod[i];
  delete [] auxxyzv;  delete [] auxnod;
}

void print_confile (const char *file,long nnod,long nel,double **auxxyzv,long **auxnod,long dim)
{
  long i,j,nbgn,nbgtr,nbgtet;
  ivector newnum(nnod+nel);
  FILE *con;

  con = fopen (file,"w");                    if (con==NULL) fprintf (stderr,"\n .con file has not been specified.\n Try it again!\n");

  fprintf (con,"00000000010000000002\n");
  fprintf (con,"0000000001000000000200000000030000000004\n\n");

  nbgn = 0;
  for (i=0;i<nnod+nel;i++){
    if (auxxyzv[i] != NULL){
      fprintf (con,"%6ld%15.4f%15.4f%15.4f%15.4f\n",++nbgn,auxxyzv[i][0],auxxyzv[i][1],auxxyzv[i][2],auxxyzv[i][3]);
      newnum[i] = nbgn; 
    }
  }

  fprintf (con,"\n");

  nbgtr = nbgtet = 0;
  if (dim == 2)
    for (i=0;i<nel;i++)
      for (j=0;j<auxnod[i][0];j++)
	fprintf (con,"%6ld%6ld%6ld%6ld\n",++nbgtr,newnum[auxnod[i][j*3+1]-1],newnum[auxnod[i][j*3+2]-1],newnum[auxnod[i][j*3+3]-1]);
  if (dim == 3)
    for (i=0;i<nel;i++)
      for (j=0;j<auxnod[i][0];j++)
	fprintf (con,"%6ld%6ld%6ld%6ld%6ld\n",++nbgtet,newnum[auxnod[i][j*3+1]-1],newnum[auxnod[i][j*3+2]-1],newnum[auxnod[i][j*3+3]-1],newnum[auxnod[i][j*3+4]-1]);

  rewind (con);
  fprintf (con,"%10d%10d\n",3,1);
  fprintf (con,"%10ld%10d%10ld%10ld\n",nbgn,0,nbgtr,nbgtet);

  fclose (con);
}


// ************************************
//          OTHER FUNCTIONS
// ************************************
void fprintf_matrix (FILE *stream, matrix &mx, char s[])
{
  long m = mx.m;
  long n = mx.n;   fprintf(stream,"\n %s(%ld,%ld):",s,m,n);
  long i,j;
  for (i=0;i<m;i++){
    for(j=0;j<n;j++){
      if (j%18==0)    fprintf(stream," \n");
      fprintf(stream,"%12.4f",mx[i][j]);
    }
    fprintf(stream,"  **");
  }
}

void fprintf_vector (FILE *stream,vector &v,char s[],long c)
{
  long n = v.n - 1;
  if (c==0) c = 1000000;
  fprintf (stream,"\n %s(%ld):\n [ 1]: ",s,v.n);
  for (long i=0; i<n; i++){
    fprintf (stream,"%15.8f",v[i]);
    if ((i+1)%c==0) fprintf(stream,"\n [%2ld]: ",i+2);
  }
  fprintf (stream,"%15.8f",v[n]);
}

void fprintf_ivector (FILE *stream,ivector &v,char s[],long c)
{
  long n = v.n - 1;
  if (c==0) c = 1000000;
  fprintf (stream,"\n %s(%ld):\n [ 1]: ",s,v.n);
  for (long i=0; i<n; i++){
    fprintf (stream,"%5ld",v[i]);
    if ((i+1)%c==0) fprintf(stream,"\n [%2ld]: ",i+2);
  }
  fprintf (stream,"%5ld",v[n]);
}

void fprintf_d_1D (FILE *stream,double *p,long p_n,char s[],long c)
{
  long n = p_n - 1;
  if (c==0) c = 1000000;
  fprintf (stream,"\n %s(%ld):\n [ 1]: ",s,p_n);
  for (long i=0; i<n; i++){
    fprintf (stream,"%14.6f",p[i]);
    if ((i+1)%c==0) fprintf(stream,"\n [%2ld]: ",i+2);
  }
  fprintf (stream,"%14.6f",p[n]);
}

void fprintf_l_1D (FILE *stream,long *p,long p_n,char s[],long c)
{
  long n = p_n - 1;
  if (c==0) c = 1000000;
  fprintf (stream,"\n %s(%ld):\n [ 1]: ",s,p_n);
  for (long i=0; i<n; i++){
    fprintf (stream,"%5ld",p[i]);
    if ((i+1)%c==0) fprintf(stream,"\n [%2ld]: ",i+2);
  }
  fprintf (stream,"%5ld",p[n]);
}

void fprintf_l_2D (FILE *stream,long **p,long p_n,long p_m,char s[],long c)
{
  long i,j;

  for (i=0;i<p_n;i++){
    if (c==0) c = 1000000;
    fprintf (stream,"\n %s[%2ld]:",s,i);
    for (j=0;j<p_m;j++){
      fprintf (stream,"%5ld",p[i][j]);
      if ((j+1)%c==0 && (j+1)!=p_m) fprintf(stream,"\n   ");
    }
  }
}


/**
   Function detects count of elements containing linear edge defined by two nodes.
   Array gt->adjelnod has to be in ascending order.
   count == 1 => the edge is on the boundary of domain
   count == 2 => the edge is inside of domain
   
   @param gt - gtopology of domain
   @param node1,node2 - nodes of the edge
   
   @return - count of elements containing the edge
   
   created 19.8.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
long edge_position (gtopology *gt,long node1,long node2)
{
  long c,a1,a2,n1,n2,*p1,*p2;
  
  a1=a2=c=0;
  n1 = gt->nadjelnod[node1];      p1 = gt->adjelnod[node1];
  n2 = gt->nadjelnod[node2];	 p2 = gt->adjelnod[node2];
  
  for (;;)
    if (p1[a1] > p2[a2])        { a2++;             if (a2 == n2)                        return (c); }
    else  if (p1[a1] == p2[a2]) { a1++; a2++; c++;  if (c  == 2 || a2 == n2 || a1 == n1) return (c); }
          else                  { a1++;             if (a1 == n1)                        return (c); }
}

/**
   Function detects count of elements containing linear surface defined by three nodes.
   Array gt->adjelnod has to be in ascending order.
   count == 1 => the surface is on the boundary of domain
   count == 2 => the surface is inside of domain
   
   @param gt - gtopology of domain
   @param node1,node2,node3 - nodes of the surface
   
   @return - count of elements containing the surface
   
   created 19.8.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
long surface_position (gtopology *gt,long node1,long node2,long node3)
{
  long c,a1,a2,a3,n1,n2,n3,*p1,*p2,*p3;
  
  a1=a2=a3=c=0;
  n1 = gt->nadjelnod[node1];      p1 = gt->adjelnod[node1];
  n2 = gt->nadjelnod[node2];	 p2 = gt->adjelnod[node2];
  n3 = gt->nadjelnod[node3];	 p3 = gt->adjelnod[node3];
  
  for (;;)
    if (p1[a1] > p2[a2])                    { a2++;                   if (a2 == n2)                                    return (c); }
    else  if (p1[a1] < p2[a2])              { a1++;                   if (a1 == n1)                                    return (c); }
          else  if (p1[a1] > p3[a3])        { a3++;                   if (a3 == n3)                                    return (c); }
                else  if (p1[a1] == p3[a3]) { a1++; a2++; a3++; c++;  if (c  == 2 || a1 == n1 || a2 == n2 || a3 == n3) return (c); }
                      else                  { a1++; a2++;             if (a1 == n1 || a2 == n2)                        return (c); }
}

/**
   created  21.10.2003, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
long adjelem2edge (gtopology *gt,long node1,long node2,long eid)
{
  long a1,a2,n1,n2,*p1,*p2;
  
  a1=a2=0;
  n1 = gt->nadjelnod[node1];      p1 = gt->adjelnod[node1];
  n2 = gt->nadjelnod[node2];	 p2 = gt->adjelnod[node2];
  
  for (;;)
    if      (p1[a1] > p2[a2]) { a2++;       if (a2 == n2)             return (-1); }
    else if (p1[a1] < p2[a2]) { a1++;       if (a1 == n1)             return (-1); }
    else if (p1[a1] == eid)   { a1++; a2++; if (a2 == n2 || a1 == n1) return (-1); }
    else return (p1[a1]);
}

long opposite_node (gtopology *gt,long *nod,long eid)
{
  long i,j,k;
  long *nodes;
  
  nodes = NULL;
  
  for (i=0;i<gt->nadjelnod[nod[0]];i++)
    for (j=0;j<gt->nadjelnod[nod[1]];j++)
      for (k=0;k<gt->nadjelnod[nod[2]];k++)
	if ( gt->adjelnod[nod[0]][i] == gt->adjelnod[nod[1]][j] && gt->adjelnod[nod[1]][j] == gt->adjelnod[nod[2]][k] && gt->adjelnod[nod[0]][i] != eid){
	  nodes = gt->gelements[gt->adjelnod[nod[0]][i]].nodes;
	  i = j = k = 999999;
	}
  
  if (nodes == NULL)                                                     return (-1);
  if (nodes[0] != nod[0] && nodes[0] != nod[1] && nodes[0] != nod[2])    return (nodes[0]);
  if (nodes[1] != nod[0] && nodes[1] != nod[1] && nodes[1] != nod[2])    return (nodes[1]);
  if (nodes[2] != nod[0] && nodes[2] != nod[1] && nodes[2] != nod[2])    return (nodes[2]);
                                                                         return (nodes[3]);
}

/**
   
   created 19.8.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
   
   long common_numbers (long n1,long *p1,long n2,long *p2)
   {
   long c=0;
   register int i,j;
   
   for (i=0;i<n1;i++)
   for (j=0;j<n2;j++)
   if (p1[i]==p2[j]){
   c++;
   break;
   }
   
   return c;
   }
*/
