#include "gnode.h"
#include "gtopology.h"


gnode::gnode (void)
{
  //  number of degrees of freedom
  ndofn=0;
  //  array of code numbers
  cn=NULL;
  //  node coordinates
  x=0.0;  y=0.0;  z=0.0;
  
  //  it is defined with respect to application of sparse direct solver
  //  see function auxdatsparsesolver in gmatrix.cpp
  ai=-1;
  
  //  general time function
  tgf=NULL;
}

gnode::~gnode (void)
{
  delete [] cn;
  delete [] tgf;
}

/**
   functions reads basic informations
   
   @param in - input stream
   
   JK
*/
void gnode::read (XFILE *in)
{
  long i;
  //  coordinates
  xfscanf (in,"%lf %lf %lf",&x,&y,&z);
  
  //  code numbers
  xfscanf (in,"%ld",&ndofn);
  cn = new long [ndofn];
  for (i=0;i<ndofn;i++){
    cn[i]=1;
  }
  
}

/**
   function reads constraints from input file
   
   @param in - input stream
   
   JK
*/
void gnode::constr (long dofcontr,XFILE *in)
{
  long i;
  
  //  classical approach, constraints are read
  if (dofcontr==0){
    for (i=0;i<ndofn;i++){
      xfscanf (in,"%ld",cn+i);
    }
  }
  
  //  type of functions for DOFs control
  if (dofcontr==1){
    tgf = new long [ndofn];
    for (i=0;i<ndofn;i++){
      xfscanf (in,"%ld",tgf+i);
      tgf[i]--;
    }
  }
}

/**
   function returns number of degrees of freedom of the node
   
   JK
*/
long gnode::give_ndofn ()
{
  return ndofn;
}

/**
   function returns number of required degree of freedom
   
   @param m - required component
   
   JK
*/
long gnode::give_dof (long m)
{
  if (m>=ndofn || m<0){
    print_err("wrong number of component is required",__FILE__,__LINE__,__func__);
  }
  return cn[m];
}

/**
   function locates number of DOF to required position
   
   @param m - number of component
   @param num - number of DOF which will be saved
   
   JK
*/
void gnode::save_dof (long m,long num)
{
  if (m>=ndofn || m<0){
    print_err("wrong number of component is required",__FILE__,__LINE__,__func__);
  }
  cn[m]=num;
}

/**
   function computes distance of first point (=this node) and second point defined by 'c'
   
   @param dim - dimension of problem
   @param c - cartesian coordinates of second point
   
   created  8.6.2003, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
double gnode::distance2 (long dim,const double *c)
{
  if (dim == 2)  return ((x-c[0])*(x-c[0]) + (y-c[1])*(y-c[1]));
                 return ((x-c[0])*(x-c[0]) + (y-c[1])*(y-c[1]) + (z-c[2])*(z-c[2]));
}


/**
   function changes DOFs
   
   @param gf - time function
   @param time - actual time
   
   6.2.2006, JK
*/
void gnode::update_dofs (gfunct *gf,double time)
{
  long i,j,k;
  
  for (i=0;i<ndofn;i++){
    //  time function id
    j=tgf[i];
    //  time function at actual time
    k=gf[j].getval_long (time);
    
    cn[i]=k;
  }
}

/**
   function changes DOFs
   
   @param gf - time function
   @param time - actual time
   @param prev_time - previous time
   
   6.2.2006, JK
*/
long gnode::search_newdofs (gfunct *gf,double time,double prev_time)
{
  long i,j,k,l,nch;
  
  //  number of changes
  nch=0;
  
  if (time>prev_time){
    for (i=0;i<ndofn;i++){
      //  time function id
      j=tgf[i];
      //  time function at actual time
      k=gf[j].getval_long (time);
      //  time function at previous time
      l=gf[j].getval_long (prev_time);
      
      if (k==l || l>0){
	cn[i]=0;
        if (k!=l)
          nch++;
      }
      else{
	cn[i]=k;
	nch++;
      }
    }
  }
  else{
    //  initial time step
    //  time = starting_time
    for (i=0;i<ndofn;i++){
      //  time function id
      j=tgf[i];
      //  time function at actual time
      k=gf[j].getval_long (time);
      
      cn[i]=k;
      nch++;
    }
  }
  
  return nch;
}

