#ifndef GTOPOLOGY_H
#define GTOPOLOGY_H

#include <stdio.h>
#include "iotools.h"
#include "gnode.h"
#include "gelement.h"
#include "lgnode.h"
#include "gphase.h"
#include "floatsub.h"
#include "itablefunct.h"
#include "galias.h"
#include "gedge.h"
#include "endnode.h"
#include "seqtop.h"
#include "ordering.h"

//#include "gmatrix.h"
//#include "gsurf.h"
class gmatrix;

/**
   class gtopology
   
   this class contains general topological data of the problem
   
   JK
*/
class gtopology
{
 public:
  gtopology ();
  ~gtopology ();
  
  void initiate (long **icn,long *nc,long m);
  void alloc_nodes (long m);
  void alloc_elements (long m);
  void alloc_lnodes (long m);
  void alloc_phases (long m);//5.4.2005 TKr

  //void alloc_lelements (long m);
  //long give_nl (long eid);
  long give_ndofn (long nid);
  long give_nmult (long lnid);
  long give_ndofe (long eid);
  long give_gndofe (long eid);
  long give_ndofedge (long edid);
  long give_nnedge (long edid);
  long give_nne (long eid);
  long give_cne (long eid);
  long give_dof (long nid,long n);
  void save_dof (long nid,long n,long num);
  long give_ldof (long nid,long lid,long n);
  void save_ldof (long nid,long lid,long n,long num);
  void give_nodes (long eid,ivector &nod);
  void give_node_coord1d (vector &x,long eid);
  void give_node_coord2d (vector &x,vector &y,long eid);
  void give_node_coord2dxz (vector &x,vector &z,long eid);
  void give_node_coord3d (vector &x,vector &y,vector &z,long eid);
  void give_code_numbers (long eid,long *cn);
  void give_gcode_numbers (long eid,long *cn);
  void give_node_code_numbers (long nid,long *cn);
  void give_mult_code_numbers (long nid,long lid,long *cn);
  void give_edge_code_numbers (long edid,long fln,long *ncn1,long *ncn2,long *mcn);
  void give_endnode_code_numbers (long enid,long *ncn1,long *ncn2,long *mcn);

  gelemtype give_elem_type (long eid);
  void give_end_nodes (long eid,long *nodes);
  void give_edge_nodes (long eid,long edid,long  *nodes);
  void give_surf_nodes (long eid,long surfid,long *nodes);

  long give_nen (long eid);
  long give_ned (long eid);
  long give_nned (long eid);
  long give_nsurf (long eid);
  long give_nnsurf (long eid);
  long give_degree (long eid);
  
  void backup_cn ();
  //long codenum_generation (slesolv *ssle,FILE *out);
  long codenum_generation (FILE *out);
  long gencodnum ();
  long schur_ordering ();
  long saddlepoint_ordering (long ns,long *nnsd,long **ltg);
  
  void edge_localization (gmatrix *gm);
  
  
  void node_mark (long n);

  void comptop (gtopology *gt1,gtopology *gt2);
  void cndistr (gtopology *gt1,gtopology *gt2);
  void adjacelem (FILE *out);
  void adjacnodes (FILE *out);
  void adjacelem_edge (FILE *out);
  void adjacelem_surf (FILE *out);
  void adjacnodes_edge (FILE *out);
  void adjacnodes_surf (FILE *out);
  void backup_codnum (void);
  void restore_codnum (void);
  void write_nodes(FILE *out);
  void unodelnode ();
  void initiate_elemmult ();
  void comp_domain_sizes ();
  void give_domain_sizes (double *sizes);
  

  void read_gf (XFILE *in);
  void print_gf (FILE *out);

  void auxinf_init ();
  void search_newdofs (double time);

  //void restore_leso ();

  void print_time_functions (FILE *kon,double time);

  //void gener_phases (long ncp,long *nan,long *nae);
  
  //  3.11.2006
  void update_elem (double time);
  void update_nodes ();
  long search_newelem (double time,double prev_time);
  long search_newdofs (double time,double prev_time);
  void update_dofs (double time);
  //void update_mesh (double time,double prev_time,long &ncn,long &nce);
  
  void lneso_init ();  


  
  ///  saddle point problems
  ///  e.g. hemivariational inequalities or discontinuities in transport processes
  void end_nodes (FILE *out);
  void endnodes_auxprint (FILE *out);
  void alloc_endnode_cn ();
  void endnodes_localization (gmatrix *gm);
  void enodes_allelem (FILE *out);
  
  void edges (FILE *out);
  void edge_dirvect ();
  void edge_normvect ();
  void edgenode_sorting ();
  void prev_next_edges ();
  void edge_series (FILE *out);
  void edge_elem (FILE *out);
  void edge_allelem (FILE *out);
  void normvectorient ();
  //void cnmodif (FILE *out);
  void edges_auxprint (FILE *out);
  
  void alloc_edge_cn ();

  void auto_subdom_detection (FILE *out);
  long codenum_multip ();
  void mult_localization (gmatrix *gm);

  void domdecomp ();
  void create_ltg (FILE *out);
  void read_seq_top (XFILE *in);
  
  ///  renumbering
  void cuthill_mckee_renumb (FILE *out);

  void writePriorityQueue(int *fronta);
  void sloan_renumb (FILE *out);
  void shell_sort( int *array, int arrayLength);
  void lastLevelOfRootedStructure(int start, int *velikost, int *maxSirka, int*hloubka, int *vzdalenost, int **lastLevel);
  int findMinimumDegree();
  void removeRepetitiousDegrees(int **pole, int *velikost);
  void generateInitialList(int start, int **seznam, int * velikostSeznamu, int *hloubkaSeznamu, int *maxSirka);
  void findPseudoPheripheral(int *startUzel, int *cilUzel);
  
  enum Status{
    ACTIVE = 1, 
    INACTIVE = 2, 
    PREACTIVE= 3,
    POSTACTIVE = 4
  };

  gnode *gnodes;
  gelement *gelements;
  lgnode *lgnodes;
  gphase *gphases;

  
  ///  number of nodes
  long nn;
  ///  number of elements
  long ne;
  ///  number of layered nodes
  long nln;
  ///  number of unknowns defined on the mesh
  long ndof;
  ///  number of internal DOFs
  long nidof;
  ///  number of boundary/interface DOFs
  long nbdof;
  ///  number of unknowns in saddle point problem
  long nsad;
  ///  number of nodes which are switched on
  ///  it is used in problems with changing number of nodes and elements
  long nnso;
  ///  number of elements which are switched on
  ///  it is used in problems with changing number of nodes and elements
  long neso;
  ///  number of subdomains/aggregates
  long ns;
  ///  number of phases
  long nph;
  
  
  
  
  


  /**
     indicator of functions for DOFs control
     dofcontr=0 - classical approach
     dofcontr=1 - DOFs are controlled by function
  */
  long dofcontr;

  /**
     state of code numbers = state of equation numbers
     this variable is used especially in problems with changing number of nodes, elements and DOFs
     DOFs are switched on and off during computation
     
     cnstate=0 - code numbers are not generated, they are not available
     cnstate=1 - code numbers are generated, they are available
  */
  long cnstate;
  
  /**
     array contains reordered node numbers
     ordering[i]=j - the i-th reordered node has number j before reordering
  */
  long *ordering;
  ///  type of node renumbering
  noderenumb nodren;
  
  ///  number of functions for DOFs control
  long ngf;
  ///  functions for DOFs control
  gfunct *gf;
  
  ///  type of generator of code numbers
  ///  cngen = 1 - the classical code number generation
  ///  cngen = 2 - the Schur system of code number generation
  ///              internal DOFs are ordered first, interface/boundary DOFs last
  long cngen;

  ///  sizes of solved domain
  double *domsizes;




  // ***********************
  //  ELEMENTS - NODES MAP
  // ***********************

  ///  array of numbers of adjacent elements to nodes
  ///  nadjelnod[i]=j - the i-th node is shared by j elements
  long *nadjelnod;

  ///  array of adjacent elements to nodes
  ///  adjelnod[i][j]=k - the j-th element adjacent to the i-th node has number k
  long **adjelnod;
  
  // **************************
  //  ELEMENTS - ELEMENTS MAP
  // **************************
  
  ///  array of numbers of adjacent elements to elements
  ///  nadjelel[i]=j - there are j adjacent elements to the i-th element
  ///  array has to be sorted, there are multiple contributions
  long *nadjelel;

  ///  array of adjacent elements to elements
  ///  adjelel[i][j]=k - the j-th element adjacent to the i-th element has number k
  ///  array has to be sorted, there are multiple contributions
  long **adjelel;

  // ***********************
  //  NODES - NODES MAP
  // ***********************

  ///  numbers of adjacent nodes to nodes
  long *nadjnodnod;
  ///  array of adjacent nodes to nodes
  long **adjnodnod;
  
  
  // ************************************
  //  ARRAYS FOR CORNER NODES DETECTION
  // ************************************
  
  ///  nadjacedgeelem - array contains list of numbers of adjacent elements which share one edge
  ///  nadjacedgeelem[i][j] = k - the j-th edge on the i-th element has k neighbouring elements
  long **nadjacedgeelem;

  ///  adjacedgeelem - array contains list of adjacent elements which share one edge
  ///  adjacedgeelem[i][j][k] = l - the k-th neighbour element of the j-th edge on the i-th element has number l
  long ***adjacedgeelem;

  ///  nadjacsurfaceelem - array contains list of numbers of adjacent elements which share one surface
  ///  nadjacsurfelem[i][j] = k - the j-th surface on the i-th element has k neighbouring elements
  long **nadjacsurfelem;
  
  ///  adjacsurfaceelem - array contains list of adjacent elements which share one surface
  ///  adjacsurfaceelem[i][j][k] = l - the k-th neighbour element of the j-th surface on the i-th element has number l
  long ***adjacsurfelem;
  
  ///  nadjacnodessurf - array of numbers of surfaces which are shared by nodes
  ///  nadjacnodessurf[i]=j - the i-th node belongs to j surfaces
  long *nadjacnodessurf;

  ///  n_nnsurf - array of numbers of nodes on particular surfaces
  ///  n_nnsurf[i][j]=k - the j-th surface adjacent to the i-th node contains k nodes
  long **n_nnsurf;

  ///  adjacnodessurf - list of adjacent nodes to nodes which share a surface
  ///  adjacnodessurf[i][j][k]=l - the k-th node of the j-th surface adjacent to the i-th node has number l
  long ***adjacnodessurf;
  
  ///  n_nned - array of numbers of nodes on particular edges
  ///  n_nned[i][j]=k - the j-th edge connected to the i-th node contains k nodes
  //long **n_nned;
  
  ///  list of numbers of adjacent nodes connected to node by an edge
  ///  nadjacnodesedge[i]=j - the i-th node has j adjacent nodes connected by an edge
  long *nadjacnodesedge;

  ///  list of adjacent nodes with respect to edges
  ///  adjacnodesedge[i][j]=k the j-th adjacent nodes connected by an edge to node i has number k
  long **adjacnodesedge;

  
  

  
  
  ///  array with code numbers backup
  long **bckcn;
  ///  usual node - layered node correspondation
  ///  important for layered problems
  long *unln;
  ///  usual node - number of layer correspondation
  ///  important for layered problems
  long *unnl;
  ///  correspondence of code numbers between two generalized topologies
  long *cngtopcorr;
  
  ///  object of floating subdomains
  floatsub flsub;
  
  ///  list of nodes which are switched on
  ///  it is motivated by problem with changing number of nodes and elements
  ///  lnso[i]=0 - the i-th node is switched off
  ///  lnso[i]=1 - the i-th node is switched on
  ///
  ///  all components of array lnso are equal to 1 for all types of problems
  ///  except problems with changing number of nodes and elements
  long *lnso;
  
  ///  list of elements which are switched on
  ///  it is motivated by problem with changing number of nodes and elements
  ///  leso[i]=0 - the i-th element is switched off
  ///  leso[i]=1 - the i-th element is switched on
  ///
  ///  all components of array leso are equal to 1 for all types of problems
  ///  except problems with changing number of nodes and elements
  long *leso;
  ///  backup array for leso components
    //long *buleso;
    
  ///  array marking nodes in problems with changing number of nodes, DOFs and elements
  long *nm;
  
  

  ///  TOOLS FOR PROBLEMS WITH HEMIVARIATIONAL INEQUALITIES

  ///  number of general edges
  long nged;
  ///  number of series
  long nser;

  ///  series of edges
  ///  edgeser[i]=j - the i-th edge is in the j-th serie
  long *edgeser;
  
  ///  number of edges in serie
  ///  nedser[i]=j - the i-th serie contains j edges
  long *nedser;
  
  ///  list of edge numbers in series
  ///  edgelist[i][j]=k - the j-th edge in the i-th serie has number k
  long **edgelist;

  gedge *gedges;
  //gsurf *gs;

  ///  number of end nodes
  long nen;
  ///  end nodes
  endnode *endnodes;
  
  seqtop *stop;
  ///  indicator of sequential topology reading
  ///  rst=0 - sequential topology is not read
  ///  rst=1 - sequential topology is read
  ///  rst=2 - sequential topology is read in the form of Boolean matrices
  long rst;
  
  
};

#endif
