#include "siftop.h"
#include "galias.h"
//#include "siftop_element_types.h"
//#include "cmlfile.h"
#include <stdlib.h>
#include <string.h>

const gtypel operator ++(gtypel &a, int)
{
  gtypel old;
  switch (a)
  {
    case isolinear1d:
      old = a;
      a = isoquadratic1d;
      return old;
    case isoquadratic1d:
      old = a;
      a = trianglelinear;
      return old;
    case trianglelinear:
      old = a;
      a = trianglequadratic;
      return old;
    case trianglequadratic:
      old = a;
      a = isolinear2d;
      return old;
    case isolinear2d:
      old = a;
      a = isoquadratic2d;
      return old;
    case isoquadratic2d:
      old = a;
      a = tetrahedronlinear;
      return old;
    case tetrahedronlinear:
      old = a;
      a = tetrahedronquadratic;
      return old;
    case tetrahedronquadratic:
      old = a;
      a = pyramidelinear;
      return old;
    case pyramidelinear:
      old = a;
      a = pyramidequadratic;
      return old;
    case pyramidequadratic:
      old = a;
      a = vedgelinear;
      return old;
    case vedgelinear:
      old = a;
      a = vedgequadratic;
      return old;
    case vedgequadratic:
      old = a;
      a = isolinear3d;
      return old;
    case isolinear3d:
      old = a;
      a = isoquadratic3d;
      return old;
    case isoquadratic3d:
      old = a;
      a = all_element_types;
      return old;
    default:
      print_err("increasing of invalid gtypel value is required", __FILE__, __LINE__, __func__);
  }
  return all_element_types;
}

/**
  Constructor setups all data members to zero values
  
  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
snode::snode()
{
  x = y = z = 0.0;
  nprop = 0;
  entid = NULL;
  prop  = NULL;
}



/**
  Destructor deletes dynamically allocated data members
  
  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
snode::~snode()
{
  delete [] entid;
  delete [] prop;
}


/**
  This method allocates array properties which size is given by num_prop

  @param num_prop - number of nodes of given element i.e. size of node array

  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
void snode::alloc(long num_prop)
{
  nprop = num_prop;
  entid = new entityp[nprop];
  prop  = new long[nprop];
}



/**
  This method searches array of properties for property aprop with given entity type ent

  @param prop - searched property number 
  @param ent  - general entity type whose property is searched

  @retval 0 - searched property was not found on given entity type
  @retval 1 - searched property was found

  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
long snode::searchprop(long aprop, gentity ent)
{
  long i;
  for(i=0; i<nprop; i++)
  {
    switch(ent)
    {
      case gvertex:
      case gcurve:
      case gregion: 
        if ((int(ent) == int(entid[i])) && (prop[i] == aprop))   return 1;
        break;
      case gsurface:
        if (((entid[i] == esurface) || (entid[i] == epatch) || 
             (entid[i] == eshell)) && (prop[i] == aprop))    return 1;
	break;
      default:
        print_err("Unknown type of entity is required", __FILE__, __LINE__, __func__);
        abort();
    }
  }
  return 0;
}



/**
  Constructor setups all data members to zero values
  
  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
selement::selement()
{
  type = gtypel(0);
  nne = 0L;

  ned =  tnned = 0L;
  nned     = NULL;
  propedg  = NULL;
  edgenod  = NULL;

  nsurf = tnnsurf = 0;
  nnsurf   = NULL;
  surfnod  = NULL;
  propsurf = NULL;  

  nodes    = NULL;
  prop     = 0L;
}



/**
  Destructor deletes dynamically allocated data members
  
  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
selement::~selement()
{
  long i;
  delete [] nned;
  delete [] nnsurf;
  delete [] nodes;
  delete [] propedg;
  delete [] propsurf;
  for(i=0; i<ned; i++)
    delete [] edgenod[i];
  for(i=0; i<nsurf; i++)
    delete [] surfnod[i];
  delete [] edgenod;
  delete [] surfnod;
}



/**
  This method allocates array nodes depending on the element type.
  Element type is given by data member type.

  @param alloc_bp - controls allocation of boundary properties arrays on elements 
                    (1 = wil be allocated/ 0 = will not be allocated)

  @retval 0 : on success allocation
  @retval 1 : unknown type is required


  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
long selement::alloc(long alloc_bp)
{
  long i;
  switch (type)
  {
    case isolinear1d:
      nne   = 2;
      ned   = 1;
      nned  = new long[ned];
      edgenod = new long*[ned];
      for(i=0; i<ned; i++)
      {
        nned[i] = 2;
        edgenod[i] = new long[nned[i]];
      }
      edgenod[0][0] = 0;
      edgenod[0][1] = 1;
      nsurf = 0;
      tnned = 2;
      tnnsurf = 0;
      break;
    case isoquadratic1d:
      nne   = 3;
      ned   = 1;
      nned  = new long[ned];
      edgenod = new long*[ned];
      for(i=0; i<ned; i++)
      {
        nned[i] = 3;
        edgenod[i] = new long[nned[i]];
      }
      edgenod[0][0] = 0;
      edgenod[0][1] = 1;
      edgenod[0][2] = 2;
      nsurf = 0;
      tnned = 3;
      tnnsurf = 0;
      break;
    case trianglelinear:
      nne   = 3;
      ned   = 3;
      nsurf = 1;
      nned  = new long[ned];
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 2;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0] = 0;  edgenod[0][1] = 1;
      edgenod[1][0] = 1;  edgenod[1][1] = 2;
      edgenod[2][0] = 2;  edgenod[2][1] = 0;
      nnsurf = new long[1];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 3;
        surfnod[i] = new long[nnsurf[i]];        
      }
      surfnod[0][0] = 0;  surfnod[0][1] = 1;  surfnod[0][2] = 2;
      tnned = 6;
      tnnsurf = 3; 
      break;
    case trianglequadratic:
      nne   = 6;
      ned   = 3;
      nsurf = 1;
      nned  = new long[ned];
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 3;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0] = 0;  edgenod[0][1] = 1;  edgenod[0][2] = 3;
      edgenod[1][0] = 1;  edgenod[1][1] = 2;  edgenod[1][2] = 4;
      edgenod[2][0] = 2;  edgenod[2][1] = 0;  edgenod[2][2] = 5;
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 6;
        surfnod[i] = new long[nnsurf[i]];        
      }
      surfnod[0][0] = 0;  surfnod[0][1] = 1;  surfnod[0][2] = 2;
      surfnod[0][3] = 3;  surfnod[0][4] = 4;  surfnod[0][5] = 5;
      tnned = 9;
      tnnsurf = 6;
      break;
    case isolinear2d:
      nne   = 4;
      ned   = 4;
      nsurf = 1;
      nned  = new long[ned];
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 2;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0] = 0;  edgenod[0][1] = 1;
      edgenod[1][0] = 1;  edgenod[1][1] = 2;
      edgenod[2][0] = 2;  edgenod[2][1] = 3;
      edgenod[3][0] = 3;  edgenod[3][1] = 0;
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 4;
        surfnod[i] = new long[nnsurf[i]];        
      }
      surfnod[0][0] = 0;  surfnod[0][1] = 1;  surfnod[0][2] = 2;  surfnod[0][3] = 3;
      tnned = 8;
      tnnsurf = 4;
      break;
    case isoquadratic2d:
      nne   = 8;
      ned   = 4;
      nsurf = 1;
      nned  = new long[ned];
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 3;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0] = 0;  edgenod[0][1] = 1;  edgenod[0][2] = 3;
      edgenod[1][0] = 1;  edgenod[1][1] = 2;  edgenod[1][2] = 4;
      edgenod[2][0] = 2;  edgenod[2][1] = 0;  edgenod[2][2] = 5;
      edgenod[3][0] = 2;  edgenod[3][1] = 0;  edgenod[3][2] = 5;
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 8;
        surfnod[i] = new long[nnsurf[i]];        
      }
      surfnod[0][0] = 0;  surfnod[0][1] = 1;  surfnod[0][2] = 2;  surfnod[0][3] = 3;
      surfnod[0][4] = 4;  surfnod[0][5] = 5;  surfnod[0][6] = 6;  surfnod[0][7] = 7;
      tnned = 12;
      tnnsurf = 8;
      break;
    case tetrahedronlinear:
      nne   = 4;
      ned   = 6;
      nsurf = 4;
      nned  = new long[ned];
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 2;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0]=0;  edgenod[0][1]=1;
      edgenod[1][0]=1;  edgenod[1][1]=2; 
      edgenod[2][0]=2;  edgenod[2][1]=0; 
      edgenod[3][0]=1;  edgenod[3][1]=3; 
      edgenod[4][0]=3;  edgenod[4][1]=2; 
      edgenod[5][0]=3;  edgenod[5][1]=0; 
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 3;
        surfnod[i] = new long[nnsurf[i]];        
      }
      surfnod[0][0]=2;  surfnod[0][1]=1; surfnod[0][2]=3; 
      surfnod[1][0]=0;  surfnod[1][1]=2; surfnod[1][2]=3; 
      surfnod[2][0]=1;  surfnod[2][1]=0; surfnod[2][2]=3; 
      surfnod[3][0]=0;  surfnod[3][1]=1; surfnod[3][2]=2; 
      tnned = 12;
      tnnsurf = 12;
      break;
    case tetrahedronquadratic:
      nne   = 10;
      ned   = 6;
      nsurf = 4;
      nned  = new long[ned];
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 3;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0]=0;  edgenod[0][1]=1;  edgenod[0][2]=4;
      edgenod[1][0]=1;  edgenod[1][1]=2;  edgenod[1][2]=5;
      edgenod[2][0]=2;  edgenod[2][1]=0;  edgenod[2][2]=6;
      edgenod[3][0]=1;  edgenod[3][1]=3;  edgenod[3][2]=8;
      edgenod[4][0]=3;  edgenod[4][1]=2;  edgenod[4][2]=9;
      edgenod[5][0]=3;  edgenod[5][1]=0;  edgenod[5][2]=7;
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 6;
        surfnod[i] = new long[nnsurf[i]];        
      }
      surfnod[0][0]=2;  surfnod[0][1]=1; surfnod[0][2]=3;
      surfnod[0][3]=5;  surfnod[0][4]=8; surfnod[0][5]=9;  
      surfnod[1][0]=0;  surfnod[1][1]=2; surfnod[1][2]=3;
      surfnod[1][3]=6;  surfnod[1][4]=9; surfnod[1][5]=7;  
      surfnod[2][0]=1;  surfnod[2][1]=0; surfnod[2][2]=3; 
      surfnod[2][3]=4;  surfnod[2][4]=7; surfnod[2][5]=8;   
      surfnod[3][0]=0;  surfnod[3][1]=1; surfnod[3][2]=2; 
      surfnod[3][3]=4;  surfnod[3][4]=5; surfnod[3][5]=6;  
      tnned = 18;
      tnnsurf = 24;
      break;
    case pyramidelinear:
      nne   = 5;
      ned   = 8;
      nsurf = 5;
      nned  = new long[ned];
      for(i=0; i<ned; i++)
      {
        nned[i] = 2;
        edgenod[i] = new long[nned[i]];        
      }
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf-1; i++)
      {
        nnsurf[i] = 3;
        surfnod[i] = new long[nnsurf[i]];        
      }
      nnsurf[nsurf-1] = 4;
      surfnod[nsurf-1] = new long[nnsurf[nsurf-1]];
      tnned = 16;
      tnnsurf = 16;
      break;
    case pyramidequadratic:
      nne   = 13;
      ned   = 9;
      nsurf = 5;
      for(i=0; i<ned; i++)
      {
        nned[i] = 3;
        edgenod[i] = new long[nned[i]];        
      }
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf-1; i++)
      {
        nnsurf[i] = 6;
        surfnod[i] = new long[nnsurf[i]];        
      }
      nnsurf[nsurf-1] = 8;
      surfnod[nsurf-1] = new long[nnsurf[nsurf-1]];
      tnned = 27;
      tnnsurf = 32;
      break;
    case vedgelinear:
      nne   = 6;
      ned   = 9;
      nsurf = 5;
      for(i=0; i<ned; i++)
      {
        nned[i] = 2;
        edgenod[i] = new long[nned[i]];        
      }
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf-2; i++)
      {
        nnsurf[i] = 4;
        surfnod[i] = new long[nnsurf[i]];        
      }
      nnsurf[nsurf-2] = 3;
      nnsurf[nsurf-1] = 3;
      surfnod[nsurf-2] = new long[nnsurf[nsurf-2]];
      surfnod[nsurf-1] = new long[nnsurf[nsurf-1]];
      tnned = 18;
      tnnsurf = 18;
      break;
    case vedgequadratic:
      nne   = 15;
      ned   = 9;
      nsurf = 5;
      for(i=0; i<ned; i++)
      {
        nned[i] = 3;
        edgenod[i] = new long[nned[i]];        
      }
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf-2; i++)
      {
        nnsurf[i] = 8;
        surfnod[i] = new long[nnsurf[i]];        
      }
      nnsurf[nsurf-2] = 6;
      nnsurf[nsurf-1] = 6;
      surfnod[nsurf-2] = new long[nnsurf[nsurf-2]];
      surfnod[nsurf-1] = new long[nnsurf[nsurf-1]];
      tnned = 27;
      tnnsurf = 36;
      break;
    case isolinear3d:
      nne   = 8;
      ned   = 12;
      nsurf = 6;
      nned  = new long[ned];
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 2;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0] =0;  edgenod[0][1] =1; 
      edgenod[1][0] =1;  edgenod[1][1] =2; 
      edgenod[2][0] =2;  edgenod[2][1] =3; 
      edgenod[3][0] =3;  edgenod[3][1] =0; 
      edgenod[4][0] =0;  edgenod[4][1] =4; 
      edgenod[5][0] =1;  edgenod[5][1] =5; 
      edgenod[6][0] =2;  edgenod[6][1] =6; 
      edgenod[7][0] =3;  edgenod[7][1] =7; 
      edgenod[8][0] =4;  edgenod[8][1] =5; 
      edgenod[9][0] =5;  edgenod[9][1] =6; 
      edgenod[10][0]=6;  edgenod[10][1]=7; 
      edgenod[11][0]=7;  edgenod[11][1]=4; 
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 4;
        surfnod[i] = new long[nnsurf[i]];        
      }
      tnned = 24;
      tnnsurf = 24;
      break;
    case isoquadratic3d:
      nne   = 20;
      ned   = 12;
      nsurf = 6;
      edgenod = new long*[ned];
      surfnod = new long*[nsurf];
      for(i=0; i<ned; i++)
      {
        nned[i] = 3;
        edgenod[i] = new long[nned[i]];        
      }
      edgenod[0][0] =0;  edgenod[0][1] =1;  edgenod[0][2] = 8; 
      edgenod[1][0] =1;  edgenod[1][1] =2;  edgenod[1][2] = 9;
      edgenod[2][0] =2;  edgenod[2][1] =3;  edgenod[2][2] = 10;
      edgenod[3][0] =3;  edgenod[3][1] =0;  edgenod[3][2] = 11;
      edgenod[4][0] =0;  edgenod[4][1] =4;  edgenod[4][2] = 12;
      edgenod[5][0] =1;  edgenod[5][1] =5;  edgenod[5][2] = 13;
      edgenod[6][0] =2;  edgenod[6][1] =6;  edgenod[6][2] = 14;
      edgenod[7][0] =3;  edgenod[7][1] =7;  edgenod[7][2] = 15;
      edgenod[8][0] =4;  edgenod[8][1] =5;  edgenod[8][2] = 16;
      edgenod[9][0] =5;  edgenod[9][1] =6;  edgenod[9][2] = 17;
      edgenod[10][0]=6;  edgenod[10][1]=7;  edgenod[10][2]= 18;
      edgenod[11][0]=7;  edgenod[11][1]=4;  edgenod[11][2]= 19;
      nnsurf = new long[nsurf];
      for(i=0; i<nsurf; i++)
      {
        nnsurf[i] = 8;
        surfnod[i] = new long[nnsurf[i]];        
      }
      surfnod[0][0]=0;  surfnod[0][1]=3;  surfnod[0][2]=7;  surfnod[0][3]=4;
      surfnod[1][0]=1;  surfnod[1][1]=0;  surfnod[1][2]=4;  surfnod[1][3]=5;
      surfnod[2][0]=2;  surfnod[2][1]=1;  surfnod[2][2]=5;  surfnod[2][3]=6;
      surfnod[3][0]=3;  surfnod[3][1]=2;  surfnod[3][2]=6;  surfnod[3][3]=7;
      surfnod[4][0]=0;  surfnod[4][1]=1;  surfnod[4][2]=2;  surfnod[4][3]=3;
      surfnod[5][0]=4;  surfnod[5][1]=5;  surfnod[5][2]=6;  surfnod[5][3]=7;

      tnned = 36;
      tnnsurf = 48;
      break;
    default:
      return 1;
  }
  nodes    = NULL;
  propedg  = NULL;
  propsurf = NULL;
  if (nne)
  {
    nodes = new long[nne];
    memset(nodes, 0, sizeof(*nodes)*nne);
  }
  if (ned && alloc_bp)
  {
    propedg = new long[ned];
    memset(propedg, 0, sizeof(*propedg)*ned);
  }
  if (nsurf && alloc_bp)
  {
    propsurf = new long[nsurf];
    memset(propsurf, 0, sizeof(*propsurf)*nsurf);
  }
  return 0;
}



/**
  This method searches array of properties for property aprop with given entity type ent

  @param prop - searched property number 
  @param ent  - general entity type whose property is searched

  @retval  0 - searched property was not found on given entity type
  @retval  1 - searched property was found
  @retval -1 - properties of required entity type have not been read

  Created by Tomas Koudelka 06.2009, koudelka@cml.fsv.cvut.cz
*/
long selement::searchprop(long aprop, gentity ent)
{
  long i;
  switch(ent)
  {
    case gcurve:
      if (propedg)
      {
        for (i=0; i<ned; i++)
        {
          if (propedg[i] == aprop)
            return 1; 
        }
      }
      else
        return -1;
      break;
    case gsurface:
      if (propsurf)
      {
        for (i=0; i<nsurf; i++)
        {
          if (propsurf[i] == aprop)
            return 1; 
        }
      }
      else
        return -1;
      break;
    case gregion: 
      if (prop == aprop)   return 1;
      break;
    default:
     print_err("Unknown type of entity is required", __FILE__, __LINE__, __func__);
     abort();
  }
  return 0;
}



/**
  This constructor initializes data mebers with NULL or zero values

  Created by Tomas Koudelka 03/2008, koudelka@cml.fsv.cvut.cz
*/
siftop :: siftop (void)
{
  elements = NULL;
  nodes    = NULL;
  gnn      = NULL;
  nn       = 0;
  ne       = 0;
  memset(npet, 0, sizeof(npet));
}



/**
  This destructor deallocates memory used by data mebers

  Created by Tomas Koudelka 03.2008, koudelka@cml.fsv.cvut.cz
*/
siftop :: ~siftop(void)
{
  if (elements)
    delete [] elements;

  if (nodes)
    delete [] nodes;

  if (gnn)
    delete [] gnn;
}



/**
  This method reads topology of given domain from file, which is in the JKTK format

  @param in   - pointer to structure with opened file of input data
  @param rgnn - indicator whether global node numbers array should be read (=1) or no (=0).
                This is support for paralell computing.

  @retval 0 : on success
  @retval 1 : on error in reading of node
  @retval 2 : on error in reading of element
  @retval 3 : on error in reading of global node numbers

  @note JKTK mesh format is generated by programs gensif??? and details are in . . .

  created by Tomas Koudelka 3.2008, koudelka@cml.fsv.cvut.cz
*/
long siftop :: read(XFILE *in, long rgnn, long rebp)
{
  long i, j, eid, nid;
  char emsg[1001];
  
  //  number of nodes
  xfscanf(in, "%k%ld", "num_nodes", &nn);
  nodes = new snode[nn];
  for (i = 0; i < nn; i++)
  {
    xfscanf(in, "%k%ld", "node_id", &nid);
    if ((nid < 1) && (nid > nn))
    {
      sprintf(emsg, "node number %ld is out of range <1, %ld> (file %s, line=%ld, col=%ld)\n", nid, nn, in->fname, in->line, in->col);
      print_err(emsg, __FILE__, __LINE__, __func__);
      return (1);
    }
    nid--;
    xfscanf(in, "%k%le %k%le %k%le %k%ld", "x", &nodes[nid].x, "y", &nodes[nid].y, "z", &nodes[nid].z, "numprop", &nodes[nid].nprop);
    if (nodes[nid].nprop < 1)
    {
      sprintf(emsg, "node %ld must have at least one property (file %s, line=%ld, col=%ld)\n", nid, in->fname, in->line, in->col);
      print_err(emsg, __FILE__, __LINE__, __func__);
      return (1);
    }
    nodes[nid].alloc(nodes[nid].nprop);
    for (j=0; j < nodes[nid].nprop; j++)
      xfscanf(in, "%k%m%ld", "prop", &entityp_kwdset, nodes[nid].entid+j, nodes[nid].prop+j);
  }


  //  number of elements
  xfscanf(in, "%k%ld", "num_elements", &ne);
  elements = new selement[ne];
  for (i = 0; i < ne; i++)
  {
    xfscanf(in, "%k%ld", "elem_id", &eid);
    eid--;
    if ((eid < 0) && (eid >= ne))
    {
      sprintf(emsg, "element number %ld is out of range <1, %ld> (file %s, line=%ld, col=%ld)", eid+1, nn, in->fname, in->line, in->col);
      print_err(emsg, __FILE__, __LINE__, __func__);
      return (2);
    }
    xfscanf(in, "%k%m", "eltype", &gtypel_kwdset, &elements[eid].type);
    if (elements[eid].alloc(rebp))
    {
      sprintf(emsg, "unknown type on element %ld is required (file %s, line=%ld, col=%ld)", eid+1, in->fname, in->line, in->col);
      print_err(emsg, __FILE__, __LINE__, __func__);
    }
    xfscanf(in, "%k", "enodes");
    for (j = 0; j < elements[eid].nne; j++)
    {
      xfscanf(in, "%ld", &elements[eid].nodes[j]);
      elements[eid].nodes[j]--;
    }
    xfscanf(in, "%k%ld", "eprop", &elements[eid].prop);
    if (rebp)
    {
      if (elements[eid].ned)
        xfscanf(in, "%k", "propedg");
      for(j = 0; j < elements[eid].ned; j++)
        xfscanf(in, "%ld", elements[eid].propedg+j);
      if (elements[eid].nsurf)
        xfscanf(in, "%k", "propsurf");
      for(j = 0; j < elements[eid].nsurf; j++)
        xfscanf(in, "%ld", elements[eid].propsurf+j);
    }
  }

  
  if (rgnn==0){
    //  sequential code
    //  no additional reading
    return(0);
  }
  
  if (rgnn==1){
    //  parallel code
    gnn = new long[nn];
    memset(gnn, 0, sizeof(*gnn)*nn);
    for (i = 0; i < nn; i++){
      xfscanf(in, "%k%ld", "node_id", &j);
      if ((j < 1) && (j > nn)){
	sprintf(emsg, "node number %ld is out of range <1, %ld> (file %s, line=%ld, col=%ld)\n", j, nn, in->fname, in->line, in->col);
	print_err(emsg, __FILE__, __LINE__, __func__);
	return (3);
      }
      j--;
      xfscanf(in, "%k%ld", "glob_id", gnn+j);
    }
  }

  if (rgnn==2){
    //  sequential code, BOSS method or sequential version of the FETI method

    //  begin, added, JK, 28.10.2009
    
    //  mesh description
    //  number of subdomains/aggregates
    xfscanf (in,"%ld %ld",&meshtype,&nsd);
    
    //  number of nodes on subdomains/aggregates
    nnsd = new long [nsd];
    //  loop over the number of subdomains/aggregates
    for (i=0;i<nsd;i++){
      xfscanf (in,"%ld",nnsd+i);
    }
    //  end, JK, 28.10.2009
    
    gnn = new long[nn];
    memset(gnn, 0, sizeof(*gnn)*nn);
    for (i = 0; i < nn; i++){
      xfscanf(in, "%k%ld", "node_id", &j);
      if ((j < 1) && (j > nn)){
	sprintf(emsg, "node number %ld is out of range <1, %ld> (file %s, line=%ld, col=%ld)\n", j, nn, in->fname, in->line, in->col);
	print_err(emsg, __FILE__, __LINE__, __func__);
	return (3);
      }
      j--;
      xfscanf(in, "%k%ld", "glob_id", gnn+j);
    }
  }
  
  return(0);
}



/**
  This method copies mesh topology to general topology structure.
  It is used in preprocessors especially.

  @param top  - pointer to structure of general topology

  created by Tomas Koudelka 3.2008, koudelka@cml.fsv.cvut.cz
*/
void siftop::exporttop (gtopology *top)
{
  long i, j;
  top->alloc_nodes(nn);
  top->alloc_elements(ne);
  for (i = 0; i < nn; i++)
  {
    top->gnodes[i].x = nodes[i].x;
    top->gnodes[i].y = nodes[i].y;
    top->gnodes[i].z = nodes[i].z;
  }
  for (i = 0; i < ne; i++)
  {
    top->gelements[i].nne = elements[i].nne;
    top->gelements[i].nodes = new long[elements[i].nne];
    for (j = 0; j < elements[i].nne; j++)
      top->gelements[i].nodes[j] = elements[i].nodes[j];
  }
}



/**
  This function imports tpology in T3D format from the opened file in.

  @param in - pointer to structure with opened file of input data
  @param paral - flag for topology format used in parallel computations

  @retval -1 if topology pointer S is not set
  @retval -2 if file specified by ofname cannot be opened

  created  03.2008, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void siftop::import_t3d (XFILE *in, long paral)
{
  long i;
  long type, renum, mode, temp;
  long outmode[9];


  xfscanf(in, "%ld", &type);   // reads type of generator
  xfscanf(in, "%ld", &degree); // reads degrees of element aproximation
  xfscanf(in, "%ld", &renum);  // reads indicator of renumbering
  xfscanf(in, "%ld", &mode);   // reads output mode for elements and nodes
    
  if (paral==1)
    xfscanf(in,"%ld", &temp);
    
  // decodes mode variable and stores results in outmode array
  for (i = 0; i < 9; i++)
    outmode[i]=(mode>>i) & 1;

  xfscanf(in, "%ld", &nn);
  // allocation of memory for nodes
  if (nodes)
    delete [] nodes; 
  nodes = new snode[nn];
  if (paral)  // mesh for parallel computations is required 
  {
    if (gnn)
      delete [] gnn; 
    gnn = new long[nn];
    memset(gnn, 0, sizeof(*gnn)*nn);
  }
  switch (type) 
  // number of particular element types depends on mesh generator type 
  {
    case 3:
      // reading of number of edges, trias, tetras
      xfscanf(in, "%ld%ld%ld", npet+isolinear1d+degree-2, npet+trianglelinear+degree-2, npet+tetrahedronlinear+degree-2);
      break ;
    case 4:
      // reading of number of edges, quads and bricks
      xfscanf(in, "%ld%ld%ld", npet+isolinear1d+degree-2, npet+isolinear2d+degree-2, npet+isolinear3d+degree-2);
      break ;
    case 7:
      // reading of number of edges, trias, quads
      xfscanf(in, "%ld%ld%ld",  npet+isolinear1d+degree-2, npet+trianglelinear+degree-2, npet+isolinear2d+degree-2);
      // reading of number of tetras, pyrams, vedges and bricks
      xfscanf(in, "%ld%ld%ld%ld", npet+tetrahedronlinear+degree-2, npet+pyramidelinear+degree-2, npet+vedgelinear+degree-2, npet+isolinear3d+degree-2);
      break ;
    default:
      print_err("unknown type of generator is required", __FILE__, __LINE__, __func__);
  }
  // total number of elements
  ne=0;
  for (i = 0; i < 7; i++)
    ne+=npet[i];

  // import of section of nodes
  import_t3d_nodes(in, paral, outmode) ;

  // allocation of memory for elements
  if (elements)
    delete [] elements;
  elements = new selement[ne];

  // import of particular element types 
  import_t3d_elements(in, outmode);

  return;
}



/**
  Function imports section of nodes in the T3D format.

  @param in - pointer to structure with opened file of input data
  @param paral   - indicates whether global node numbers are read (=1) or not (=0).
  @param outmode - array with description of output type (see T3D documentation).
                   Each aray element represents one bit flag from T3D output type.

  @retval 0 : on succes
  @retval 1 : in case that node number is out of range

  created  03.2008, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long siftop::import_t3d_nodes (XFILE *in, long paral, long *outmode)
{
  long i, j, nid;
  long nent;
  entityp ent;
  long eprop;
  char emsg[1001];

  for (i = 0; i < nn; i++)
  {
    xfscanf(in, "%ld", &nid);
    if ((nid < 1) && (nid > nn))
    {
      sprintf(emsg, "node number %ld is out of range <1, %ld> (file %s, line=%ld, col=%ld)\n", nid, nn, in->fname, in->line, in->col);
      print_err(emsg, __FILE__, __LINE__, __func__);
      return (1);
    }
    nid--;
    if (paral)
      xfscanf(in, "%ld", gnn+nid);
    xfscanf(in, "%le %le %le %m %*ld %ld", &nodes[nid].x, &nodes[nid].y, &nodes[nid].z, &entityp_kwdset, &ent, &eprop);
    if (outmode[omt_sec_clasif_nod])
    {
      xfscanf(in, "%ld", &nent);
      nent++;
      nodes[nid].alloc(nent);
      nodes[nid].entid[0] = ent;
      nodes[nid].prop[0]  = eprop;
      for (j=1; j < nent; j++)
        xfscanf(in, "%m %*ld %ld", &entityp_kwdset, nodes[nid].entid+j, nodes[nid].prop+j);
    }
    else
    {
      nent = 1;
      nodes[nid].alloc(nent);
      nodes[nid].entid[0] = ent;
      nodes[nid].prop[0]  = eprop;
    }
    switch (nodes[nid].entid[0])
    {
      case evertex:
      case eregion:
        break ;
      case ecurve:
        if (outmode[omt_parameters] || outmode[omt_tangent])
          skipline(in); // skip rest of actual line
        break ;
      case esurface:
      case epatch:
      case eshell:
        if (outmode[omt_parameters] || outmode[omt_normal])
          skipline(in); // skip rest of actual line
        break ;
    }
  }
  return(0);
}



/**
  This function imports all elements from T3D file.

  @param in - pointer to structure with opened file of input data
  @param outmode - array with description of output type (see T3D documentation).
                   Each aray element represents one bit flag from T3D output type.
  @retval 0 : on success
  @retval 1 : in case that element number is out of range

  Created 03.2008, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long siftop::import_t3d_elements (XFILE *in, long *outmode)
{
  long i, j, eid;
  gtypel etype;
  char emsg[1001];
  
  if (degree == 1) 
    etype=isolinear1d;
  else
    etype=isoquadratic1d;

  for ( ; etype<all_element_types; etype++, etype++)
  {
    for (i=0; i<npet[etype-1]; i++)
    {
      elements[i].type = etype;
      if (outmode[omt_neighbourhood])
        elements[i].alloc(1);
      else
        elements[i].alloc(1);
      xfscanf(in, "%ld", &eid);
      eid--;
      if ((eid < 0) && (eid >= ne))
      {
        sprintf(emsg, "element number %ld is out of range <1, %ld> (file %s, line=%ld, col=%ld)", eid+1, nn, in->fname, in->line, in->col);
        print_err(emsg, __FILE__, __LINE__, __func__);
        return(1);
      }
      for(j=0; j<elements[eid].nne; j++)
      {
        xfscanf(in, "%ld", &elements[eid].nodes[j]);
        elements[eid].nodes[j]--;
      }

      xfscanf(in, "%*ld %*ld %ld", &elements[eid].prop);
      switch (etype)
      {
        case isolinear1d:
        case isoquadratic1d:
          break;
        case trianglelinear:
        case trianglequadratic:
        case isolinear2d:
        case isoquadratic2d:
        {  
          if (outmode[omt_neighbourhood])
            skipline(in); // skip rest of actual line
          if (outmode[omt_boundary_entities])
          {
            for (j=0; j<elements[eid].ned; j++)
              xfscanf(in, "%*ld");
            for (j=0; j<elements[eid].ned; j++)
              xfscanf(in, "%ld", elements[eid].propedg+j);
            skipline(in); // skip rest of actual line
          }
          else
            skipline(in); // skip rest of actual line
          break;
        }
        case tetrahedronlinear:
        case tetrahedronquadratic:
        case pyramidelinear:
        case pyramidequadratic:
        case vedgelinear:
        case vedgequadratic:
        case isolinear3d:
        case isoquadratic3d:
        {  
          if (outmode[omt_neighbourhood] || outmode[omt_associated_elements])
            skipline(in); // skip rest of actual line
          if (outmode[omt_boundary_entities])
          {
            for (j=0; j<elements[eid].nsurf; j++)
              xfscanf(in, "%*ld"); // skip entity id
            for (j=0; j<elements[eid].nsurf; j++)
              xfscanf(in, "%*ld"); // skip entity types
            for (j=0; j<elements[eid].nsurf; j++)
              xfscanf(in, "%ld", elements[eid].propsurf+j);
            skipline(in); // skip rest of actual line
          }
          else
            skipline(in); // skip rest of actual line
          break;
        }
        default:
          sprintf(emsg, "unknown type on element %ld is required (file %s, line=%ld, col=%ld)", eid+1, in->fname, in->line, in->col);
          print_err(emsg, __FILE__, __LINE__, __func__);
          break;
      }
    }
  }
  return(0);
}



/**
   Function prints mesh data in SSMF.
   
   created by Tomas Koudelka 3.2008, koudelka@cml.fsv.cvut.cz
*/
void siftop::print (FILE *out)
{
  long i, j;
  
  fprintf (out,"%ld\n",nn);

  for (i = 0; i < nn; i++)
  {
    fprintf (out,"%ld %15.10le %15.10le %15.10le %ld",i+1, nodes[i].x, nodes[i].y, nodes[i].z, nodes[i].nprop);
    for(j=0; j<nodes[i].nprop; j++)
      fprintf(out, " %d %ld", nodes[i].entid[j], nodes[i].prop[j]);
    fprintf(out, "\n");
  }
  
  fprintf (out,"\n%ld\n", ne);
  
  for (i = 0; i < ne; i++)
  {
    fprintf (out,"%ld %d",elements[i].nne, elements[i].type);
    for (j = 0; j < elements[i].nne; j++)
      fprintf (out," %ld",elements[i].nodes[j]+1);
    fprintf (out," %ld",elements[i].prop);
    if (elements[i].propedg)
    {
      fprintf(out, "  ");
      for (j=0; j < elements[i].ned; j++)
        fprintf (out," %ld",elements[i].propedg[j]);
    }
    if (elements[i].propsurf)
    {
      fprintf(out, "  ");
      for (j=0; j < elements[i].nsurf; j++)
        fprintf (out," %ld",elements[i].propsurf[j]);
    }
    fprintf(out, "\n");
  }
  
}



/**
  Method gets set of nodes with property prop of entity ent
  Node numbers of found nodes are stored to array setnodes

  @param prop  - searched property id 
  @param ent   - type of searched entity, in case of gsurface, 
                 the esurface, epatch and eshell are assumed to be equal to gsurface
  @param setnodes - allocated array for node numbers with given property of given entity
  
  @retval 0  - in case that the nodes with given property of given entity have the different ndof

  Created 06.2009 by Tomas Koudelka koudelka@cml.fsv.cvut.cz                
*/
long siftop::get_propent_nodes(long prop, gentity ent, long *setnodes)
{
  long i;
  for (i=0; i<nn; i++)
  {
    setnodes[i] = -1;
    if (nodes[i].searchprop(prop, ent))
      setnodes[i] = i;
  }
  return 0;
}



/**
  Method gets ndof of nodes with property prop of entity ent
  All nodes with given property of entity are searched and checked that
  they have same ndof. Node numbers of found nodes are stored to array setnodes

  @param prop  - searched property id 
  @param ent   - type of searched entity, in case of gsurface, 
                 the esurface, epatch and eshell are assumed to be equal to gsurface
  @param ndofn - array of ndofs for all nodes
  @param setnodes - allocated array for node numbers with given property of given entity
  
  @retval >0 - ndof of nodes with given property of given entity
  @retval 0  - in case that the nodes with given property of given entity have the different ndof
  @retval <0 - in case that no nodes with the given property were found

  Created 04.2008 by Tomas Koudelka koudelka@cml.fsv.cvut.cz                
*/
 long siftop::get_ndofn(long prop, gentity ent, long *ndofn, long *setnodes)
{
  long i;
  long ndof = -1;  
  for (i=0; i<nn; i++)
  {
    setnodes[i] = -1;
    if (nodes[i].searchprop(prop, ent))
    {
      setnodes[i] = i;
      if (ndof == -1)
        ndof = ndofn[i];
      else
      {
        if (ndof != ndofn[i])
          return 0;
      }
    }
  }
  return ndof;
}

