#if !defined ( __SIFTOP__ )
#define        __SIFTOP__

#include <stdio.h>
#include "galias.h"
#include "iotools.h"
#include "gtopology.h"

enum outmodetype {omt_parameters=0, omt_tangent=1, omt_normal=2, omt_boundary_entities=3, 
                  omt_associated_elements=4, omt_neighbourhood=5, omt_el_iso_type=6,
                  omt_nat_order_nod=7, omt_sec_clasif_nod=8};

const gtypel operator ++(gtypel &a, int);

struct snode
/**
  This structure groups information about node like coordinates and property number
*/
{
  double x; ///< x coordinate of node
  double y; ///< y coordinate of node
  double z; ///< z coordinate of node
  long     nprop; ///< number of assigned properties
  entityp *entid; ///< entity types
  long    *prop;  ///< property numbers of given entities

  snode();
  ~snode();
  void alloc(long num_prop);  ///< allocation of node
  long searchprop(long aprop, gentity ent); ///< searches node for given property
};

struct selement
/**
  This structure groups information about element like node numbers, property number and type of element
*/
{
  long   *nodes;    ///< pointer to array with element node numbers
  long    nne;      ///< number of nodes on element
  gtypel  type;     ///< geometric type of element
  long    ned;      ///< number of edges
  long   *nned;     ///< number of nodes on particular edges
  long    tnned;    ///< total number of nodes on all edges
  long  **edgenod;  ///< array with indeces of nodes on particular edges - edgnod[i][j] gives j-th index of node on i-th edge
  long   *propedg;  ///< property numbers of each edge
  long    nsurf;    ///< number of surfaces
  long   *propsurf; ///< property numbers of surfaces
  long   *nnsurf;   ///< number of nodes on particular surfaces
  long    tnnsurf;  ///< total number of nodes on all surfaces
  long  **surfnod;  ///< array with indeces of nodes on particular surfaces - surfnod[i][j] gives index of j-th node on i-th surface
  long    prop;     ///< property number of element

  selement();
  ~selement();

  /// allocation of memory for nodes
  long alloc(long alloc_bp);

  /// searches property aprop of given entity ent
  long searchprop(long aprop, gentity ent);
};


class siftop
/**
  This class stores all information about topology of given problem but only for auxiliary
  purposes.
*/
{
  public:
   siftop (void);
   ~siftop (void);

   /// method reads topology in JKTK format from file
   long read(XFILE *in, long rgnn, long rebp);

   /// method imports topology in T3D format
   void import_t3d(XFILE *in, long paral);

   /// method imports section of nodes from file in T3D format
   long import_t3d_nodes (XFILE *in, long paral, long *outmode);

   /// method imports section of elements from file in T3D format
   long import_t3d_elements (XFILE *in, long *outmode);

   /// method copies topologycal data to gtopology structure
   void exporttop (gtopology *top);

   /// method exports topology to a text file in JKTK format
   void print (FILE *out);

   /// method gets ndof of nodes with property prop of entity ent
   long get_ndofn(long prop, gentity ent, long *ndofn, long *setnodes);

   /// methods searches for nodes with property prop of entity ent
   void get_selnodes(long prop, gentity ent, long *selnodes);

   /// returns nodes laying on the entities ent with property prop
   long get_propent_nodes(long prop, gentity ent, long *setnodes);

   selement *elements; ///< pointer to array with elements
   snode    *nodes;    ///< pointer to array with nodes
   long      nn;       ///< number of nodes
   long      ne;       ///< number of elements
   long      npet[all_element_types-1];  ///< number of particular element types (used for t3d format)
   long      degree;   ///< degree of approximation polynom of the elements (used for t3d format)
   long     *gnn;      ///< pointer to array with global node numbers. This is support for paralell computing
       
   //  parameters for BOSS preconditioners
   ///  type of mesh description
   long meshtype;
   ///  number of subdomains/aggregates
   long nsd;
   /// number of nodes on subdomains/aggregates
   long *nnsd;
};

#endif
