#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "vector.h"
#include "iotools.h"

/**
   The constructor allocates memory form the heap to the member array a.
   
   @param n is the number of %vector elements
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
vector :: vector (long n)
{
  vector::n = n;
  a = new double[n];
  memset (a,0,n*sizeof(double));
  if (a == NULL)
  {
    fprintf(stderr,"\n\nError allocating memory for vector\n");
    fprintf(stderr," function vector::vector(long) (vector.cpp)\n");
  }
  #ifdef DEBUG_VECTOR
   Acv++;
  #endif
}

/**
   The copy constructor creates copy of the object given by the constant reference parameter.
   
   @param v is reference to the object of the vector which should be copied
   
   created  16.10.2003 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
vector :: vector (const vector &v)
{
  long i;
  if ((v.n != n) || (a == NULL))
  {
    n = v.n;
    a = new double[v.n];
    if (a == NULL)
    {
      fprintf(stderr,"\n\nError allocating memory for vector\n");
      fprintf(stderr," in constructor vector::vector() (vector.cpp)\n");
    }
  }
  for (i = 0; i < n; i++)
    a[i] = v.a[i];
  #ifdef DEBUG_VECTOR
   fprintf(stderr, "\n\nWarning copy constructor of vector is called.\n");
   fprintf(stderr, "\Please check your code and make sure you want use copy constructor.\n");
   Acv++;
  #endif
}

/**
   The destructor deallocates the memory occupied by the member array a
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
vector :: ~vector()
{
   #ifdef DEBUG_VECTOR
  if (a != NULL)
     Acv--;
  #endif
  n = 0;
  delete [] a;
  a = NULL;
}


/**
   The constructor allocates memory form the heap to the member array a.

   @param n is the number of ivector elements
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
ivector :: ivector (long n)
{
  ivector::n = n;
  a = new long[n];
  memset (a,0,n*sizeof(long));
  if (a == NULL)
  {
    fprintf(stderr,"\n\nError allocating memory for ivector\n");
    fprintf(stderr," in constructor ivector::ivector() (ivector.cpp)\n");
  }
  #ifdef DEBUG_VECTOR
   Aciv++;
  #endif
}

/**
   The copy constructor creates copy of the object given by the constant reference parameter.
   
   @param v is reference to the object of the ivector which should be copied
   
   created  16.10.2003 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
ivector :: ivector (const ivector &v)
{
  long i;
  if ((v.n != n) || (a == NULL))
  {
    n = v.n;
    a = new long[v.n];
    if (a == NULL)
    {
      fprintf(stderr,"\n\nError allocating memory for ivector\n");
      fprintf(stderr," in constructor ivector::ivector() (ivector.cpp)\n");
    }
  }
  for (i = 0; i < n; i++)
    a[i] = v.a[i];
  #ifdef DEBUG_VECTOR
   fprintf(stderr, "\n\nWarning copy constructor of ivector is called.\n");
   fprintf(stderr, "\Please check your code and make sure you want use copy constructor.\n");
   Aciv++;
  #endif
}

/**
 The destructor deallocates the memory occupied by the member array a

 created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
 modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
ivector :: ~ivector()
{
  #ifdef DEBUG_VECTOR
   if (a != NULL)
     Aciv--;
  #endif
  n = 0;
  delete [] a;
  a = NULL;
}


#ifdef DEBUG_VECTOR
 long give_acv()
 {
   return Acv;
 }
#endif


/**
   The function allocates memory from heap to the %vector's member array a.
   
   @param n   is the number of the vector elements
   @param vec is the structure for allocated vector
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long allocv(long n, vector &vec)
{
  vec.n = n;
  vec.a = new double[n];
  memset (vec.a,0,n*sizeof(double));
  if (vec.a == NULL)
  {
    fprintf(stderr,"\n\nError allocating memory for vector\n");
    fprintf(stderr," function allocv() (vector.cpp)\n");
    return (1);
  }
  #ifdef DEBUG_VECTOR
   Acv++;
  #endif
  return (0);
}

/**
   The function allocates memory from heap to the ivector's member array a.
   
   @param n   is the number of the ivector elements
   @param vec is the structure for allocated ivector
   
   @retval 0 - on success
   @retval 1 - if fails allocating memory
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long allocv(long n, ivector &vec)
{
  vec.n = n;
  vec.a = new long[n];
  memset (vec.a,0,n*sizeof(long));
  if (vec.a == NULL)
  {
    fprintf(stderr,"\n\nError allocating memory for ivector\n");
    fprintf(stderr," in function allocv() (ivector.cpp)\n");
    return (1);
  }
  #ifdef DEBUG_VECTOR
   Aciv++;
  #endif
  return (0);
}


/**
   The function copies vector given by src to dest.
   
   @param src  is the structure of source vector to copy
   @param dest is the structure of destination vector to which will be copied contents of src
   
   @b Requests :
   dest has to be setuped dimensions and allocated memory array for elements
   which is enough large to hold all contents of the vector src.

   @retval 0 : on succes
   @retval 1 : in case incompatibility sizes of the src and dest vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long copyv(const vector &src, vector &dest)
{
  if (src.n != dest.n)
  {
    fprintf(stderr,"\n\nError copying vector - incompatible size of vectors\n");
    fprintf(stderr," function copyv() (vector.cpp)\n");
    return (1);
  }
  memcpy(dest.a, src.a, sizeof(double)*src.n);
  return (0);
}

/**
   The function copies %vector given by src to dest.
   
   @param src  is the structure of source %ivector to copy
   @param dest is the structure of destination %ivector to which will be copied contents of src
   
   @b Requests :
   dest has to be setuped dimensions and allocated memory array for elements
   which is enough large to hold all contents of the %ivector src.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of the src and dest vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long copyv(const ivector &src, ivector &dest)
{
  if (src.n != dest.n)
  {
    fprintf(stderr,"\n\nError copying ivector - incompatible size of vectors\n");
    fprintf(stderr," function copyv(ivector,ivector) (ivector.cpp)\n");
    return (1);
  }
  memcpy(dest.a, src.a, sizeof(long)*src.n);
  return (0);
}

long copyv(const long *src,long *dest,long n)
{
  memcpy(dest, src, sizeof(long)*n);
  return (0);
}
/**
   The function copies double array given by src to dest.
   
   @param src  is the double array to copy
   @param dest is the double array to which will be copied contents of src
   
   @retval always zero
   
   created  3.11.2001 by JK
*/
long copyv (const double *src, double *dest, long n)
{
  memcpy(dest, src, sizeof(double)*n);
  return (0);
}


/**
   The function copies array given by src to vector dest.
   
   @param src  is source array to copy
   @param dest is the structure of destination vector which will be copied contents of src to
   
   @b Requests :
   dest has to be setuped dimensions and allocated memory array for required elements.
   Number of copied elements is given by the dest dimension

   @retval 0 : on succes
   
   created  4.10.2007 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
 */
long copyv(const double *src, vector &dest)
{
  memcpy(dest.a, src, sizeof(double)*dest.n);
  return (0);
}


/**
   The function copies vector given by src to array dest.
   
   @param src  is the structure of source vector to copy
   @param dest is the destination array which will be copied contents of src to
   
   @b Requests :
   dest has to have allocated memory which is enough large to hold all contents 
   of the vector src. Number of copied elements is given by the src dimension

   @retval 0 : on succes
   
   created  4.10.2007 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
 */
long copyv(const vector &src, double *dest)
{
  memcpy(dest, src.a, sizeof(double)*src.n);
  return (0);
}


/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %vector for the deallocated %vector
   
   @return always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long fillv(double c, vector &vec)
{
  for (long i = 0; i < vec.n; vec(i) = c, i++);
  return (0);
}

/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %ivector which should be filled
   
   @retval always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long fillv(long c, ivector &vec)
{
  for (long i = 0; i < vec.n; vec(i) = c, i++);
  return (0);
}
/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %ivector which should be filled
   
   @retval always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void fillv(long c, long *vec,long n)
{
  long i;
  
  for (i=0;i<n;i++)
    vec[i]=c;
}
/**
   The function fills vect's member array a with value c
   
   @param c is value, which will be used for filling memory
   @param vec is the structure %ivector which should be filled
   
   @retval always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void fillv(double c, double *vec,long n)
{
  long i;
  
  for (i=0;i<n;i++)
    vec[i]=c;
}


/**
   The function fills double array a with zeros
   
   @param a is array which should be filled
   
   @retval always zero
*/
long nullv (double *a,long n)
{
  memset (a,0,n*sizeof(double));
  return (0);
}
long nullv (ivector &a)
{
  memset (a.a,0,a.n*sizeof(long));
  return (0);
}


/**
   The function deallocates memory occupied by %vector vec
   
   @param vec is the structure for the deallocated %vector
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long destrv(vector &vec)
{
  #ifdef DEBUG_VECTOR
   if (a != NULL)
     Acv--;
  #endif
  vec.n = 0;
  delete [] vec.a;
  vec.a = NULL;
  return (0);
}

/**
   The function deallocates memory occupied by %ivector vec
   
   @param vec is the structure for the deallocated %ivector
   
   @retval 0
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long destrv(ivector &vec)
{
  #ifdef DEBUG_VECTOR
   if (a != NULL)
     Aciv--;
  #endif
  vec.n = 0;
  delete [] vec.a;
  vec.a = NULL;
  return (0);
}


/**
   The function adds vector given by a to %vector given by b, the result is stored in c
   
   @param a is the structure of the first added vector
   @param b is the structure of the second added vector
   @param c is the structure of the result vector
   
   @b Requests :
   a, b and c have to be same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 : on succes
   @retval 1 : in case incompatibility sizes of a, b and c vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long addv(const vector &a, const vector &b, vector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    fprintf(stderr,"\n\nError adding vectors - incompatible size\n");
    fprintf(stderr," function addv() (vector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < a.n; i++)
    c.a[i] = a.a[i] + b.a[i];

  return(0);
}

/**
   The function adds ivector given by a to ivector given by b, the result is stored in c
   
   @param a is the structure of the first added ivector
   @param b is the structure of the second added ivector
   @param c is the structure of the result ivector
   
   @b Requests :
   a, b and c should have same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long addv(const ivector &a, const ivector &b, ivector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    fprintf(stderr,"\n\nError adding ivectors - incompatible size\n");
    fprintf(stderr," function addv() (ivector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < a.n; i++)
    c.a[i] = a.a[i] + b.a[i];

  return(0);
}


/**
   The function adds double array b to double array given by a, the result is stored in a
   
   @param a is result array
   @param b is added array
   
   @retval always zero
   
   created  by JK
*/
void addv (double *a,double *b,long n)
{
  for (long i=0;i<n;i++)
    a[i]+=b[i];
}

/**
   The function adds two double arrays a and b, the result is stored in c
   
   @param a is added array
   @param b is added array
   @param c is result array
   @param n number of vector components
   
   created  by JK
*/
void addv (double *a,double *b,double *c,long n)
{
  for (long i=0;i<n;i++)
    c[i] = a[i]+b[i];
}

/**
   The function adds double array b multiplied by scalar c to double array given by a, the result is stored in a
   
   @param a is result array
   @param b is added array
   @param c is scalar
   
   @retval always zero
   
   created  by JK
*/
void addmultv (double *a,double *b,double c,long n)
{
  for (long i=0;i<n;i++)
    a[i]+=b[i]*c;
}


/**
   The function substracts vector given by b from %vector given by a, the result is stored in the c
   
   @param a is the structure of the %vector, from which is substracted
   @param b is the structure of substracted %vector
   @param c is the structure of the result %vector
   
   @b Requests :
   a, b and c have to be same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long subv(const vector &a, const vector &b, vector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    fprintf(stderr,"\n\nError substracting vectors - incompatible size\n");
    fprintf(stderr," function subv() (vector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < c.n; i++)
    c.a[i] = a.a[i] - b.a[i];

  return(0);
}

/**
   The function substracts %ivector given by b from %ivector given by a, the result is stored in the c
   
   @param a is the structure of the %ivector, from which is substracted
   @param b is the structure of substracted %ivector
   @param c is the structure of the result %ivector
   
   @b Requests :
   a, b and c should have same dimensions, c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c ivectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long subv(const ivector &a, const ivector &b, ivector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    fprintf(stderr,"\n\nError substracting ivectors - incompatible size\n");
    fprintf(stderr," function subv() (ivector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < c.n; i++)
    c.a[i] = a.a[i] - b.a[i];
      
  return(0);
}

/**
   The function substracts double array given by b from double array given by a, the result is stored in the a
   
   @param a is the array, from which is substracted
   @param b is the substracted array
   @param n is number of members of arrays
   
   @retval always zero
   
   created by JK
*/
void subv (double *a,double *b,long n)
{
  for (long i=0;i<n;i++)
    a[i]-=b[i];
}

void subv (double *a,double *b,double *c,long n)
{
  long i;
  
  for (i=0;i<n;i++){
    c[i] = a[i]-b[i];
  }
}

/**
   The function performs vector product, multiplies %vector given by a
   from right with %vector given by b, the result is stored in c
   
   @param a is the structure of the %vector, from which is multiplied
   @param b is the structure of the multiplicating %vector
   @param c is the structure of the result %vector
   
   @b Requests :
   a, b and c have to be same dimension equal 3
   c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   @retval 2 in case a and b vectors have dimension inequal 3
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long crprd(const vector &a, const vector &b, vector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    fprintf(stderr,"Error vector product - incompatible size\n");
    fprintf(stderr," function crprd() (vector.cpp)\n");
    return (1);
  }
  if ((a.n != 3) || (b.n != 3))
  {
    fprintf(stderr,"Error vector product - too large or small vector\n");
    fprintf(stderr," function crprd() (vector.cpp)\n");
    return (2);
  }
  c.a[0] = a.a[1] * b.a[2] - a.a[2] * b.a[1];
  c.a[1] = a.a[2] * b.a[0] - a.a[0] * b.a[2];
  c.a[2] = a.a[0] * b.a[1] - a.a[1] * b.a[0];
  
  return (0);
}

/**
   The function performs vector product, multiplies %ivector given by a
   from right with %ivector given by b, the result is stored in c
   
   @param a is the structure of the %ivector, from which is multiplied
   @param b is the structure of the multiplicating %ivector
   @param c is the structure of the result %ivector
   
   @b Requests :
   a, b and c should have same dimension equal 3
   c has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a, b and c vectors
   @retval 2 in case a and b vectors have dimension inequal 3
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long crprd(const ivector &a, const ivector &b, ivector &c)
{
  if ((a.n != b.n) || (b.n != c.n))
  {
    fprintf(stderr,"\n\nError ivector multiplying - incompatible size\n");
    fprintf(stderr," function crprd() (ivector.cpp)\n");
    return (1);
  }
  if ((a.n != 3) || (b.n != 3))
  {
    fprintf(stderr,"\n\nError ivector multiplying - too large ivector\n");
    fprintf(stderr," function crprd() (ivector.cpp)\n");
    return (2);
  }
  c.a[0] = a.a[1] * b.a[2] - a.a[2] * b.a[1];
  c.a[1] = a.a[2] * b.a[0] - a.a[0] * b.a[2];
  c.a[2] = a.a[0] * b.a[1] - a.a[1] * b.a[0];
  
  return (0);
}


/**
   The function performs scalar product of the %vector given by a and %vector given by b,
   the result is stored in the c.

  @param a is the structure of the scalar multiplied %vector
  @param b is the structure of the scalar multiplied %vector
  @param scalar is the real number type of double by which the result is passed

  @b Requests :
  a and b have to be same dimension
  
  @retval 0 : on succes
  @retval 1 : in case incompatibility sizes of a and b
  
  created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long scprd(const vector &a, const vector &b, double &scalar)
{
  if (a.n != b.n)
  {
    fprintf(stderr,"\n\nError scalar multiplying - incompatible size\n");
    fprintf(stderr," function scprd() (vector.cpp)\n");
    return (1);
  }
  scalar = 0.0;
  for (long i=0; i < a.n; i++)
    scalar += a.a[i] * b.a[i];
  return (0);
}

/**
   The function performs scalar product of the %ivector given by a and %ivector given by b,
   the result is stored in the c
   
   @param a is the structure of the scalar multiplied %ivector
   @param b is the structure of the scalar multiplied %ivector
   @param scalar is the integer number type of long by which the result is passed
   
   @b Requests :
   a and b should have same dimension
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of a and b
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long scprd(const ivector &a, const ivector &b, long &scalar)
{
  if (a.n != b.n)
  {
    fprintf(stderr,"\n\nError scalar multiplying ivectors - incompatible size\n");
    fprintf(stderr," function scprd() (ivector.cpp)\n");
    return (1);
  }
  scalar = 0;
  for (long i=0; i < a.n; i++)
    scalar += a.a[i] * b.a[i];
  return (0);
}


/**
   The function performs scalar product of the double array given by a and double array given by b,
   the result is stored in the c
   
   @param a is the first array
   @param b is the second array
   @param n is number of members of arrays
   
   created  by JK
*/
double ss (double *a,double *b,long n)
{
  double s = 0.0;
  for (long i=0;i<n;i++)
    s+=a[i]*b[i];
  
  return s;
}


/**
   The function multiplies %vector given by u by real constant c,
   the result is stored in the double %vector v
   
   @param c is the real number type double
   @param u is the structure of the %vector, which is multiplied
   @param v is the structure of the result %vector
   
   @b Requests :
   u and v have to be same dimensions, u has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 : on succes
   @retval 1 : in case incompatibility sizes of u and v vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(double c, const vector &u, vector &v)
{
  if (u.n != v.n)
  {
    fprintf(stderr,"Error multiplying vector by constant - incompatible size\n");
    fprintf(stderr," function cmulv() (vector.cpp)\n");
    return (1);
  }
  for (long i=0; i < u.n; i++)
    v.a[i] = c * u.a[i];

  return (0);
}

/**
   The function multiplies %ivector given by u by real constant c,
   the result is stored in the double %ivector v
   
   @param c is the real number type double
   @param u is the structure of the %ivector, which is multiplied
   @param v is the structure of the result %vector
   
   @b Requests :
   u and v should have same dimensions, u has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of u and v vectors
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(double c, const ivector &u, vector &v)
{
  if (u.n != v.n)
  {
    fprintf(stderr,"\n\nError multiplying ivector by real constant - incompatible size\n");
    fprintf(stderr," cmulv(double, ivector, vector) (ivector.cpp)\n");
    return (1);
  }
  for (long i=0; i < u.n; i++)
    v.a[i] = c * u.a[i];
  
  return (0);
}

/**
   The function multiplies %ivector given by u by integer constant c,
   the result is stored in the %vector v
   
   @param c is the integer number type long
   @param u is the structure of the %ivector, which is multiplied
   @param v is the structure of the result %ivector
   
   @b Requests :
   u and v should have same dimensions, u has to have allocated memory array for elements
   which is enough large to hold contents of the result.
   
   @retval 0 on succes
   @retval 1 in case incompatibility sizes of u and v %vector
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(long c, const ivector &u, ivector &v)
{
  if (u.n != v.n)
  {
    fprintf(stderr,"\n\nError multiplying ivector by int constant - incompatible size\n");
    fprintf(stderr," function cmulv(long, ivector, ivector) (ivector.cpp)\n");
    return (1);
  }
  for (long i=0; i < u.n; i++)
    v.a[i] = c * u.a[i];
  
  return (0);
}


/**
   The function multiplies %vector given by a with constant c,
   the result is stored in a
   
   @param c is the real number
   @param a is the structure of the %vector, which is multiplied
   
   @return always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(double c, vector &a)
{
  for (long i=0; i < a.n; i++)
    a.a[i] *= c;
  
  return (0);
}

/**
   The function multiplies %ivector given by a with integer constant c,
   the result is stored in a
   
   @param c is the integer number
   @param a is the structure of the %ivector, which is multiplied
   
   @retval 0 always
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cmulv(long c, ivector &a)
{
  for (long i=0; i < a.n; i++)
    a.a[i] *= c;
  
  return (0);
}
/**
   The function multiplies double array given by a with constant c,
   the result is stored in a
   
   @param c is the real number
   @param a is the array, which is multiplied
   @param n is number of members of arrays
   
   @return always zero
   
   created by JK
*/
void cmulv(double c,double *a,long n)
{
  for (long i=0; i < n; i++)
    a[i] *= c;
  
}

/**
   The function produces size of the %vector given by a,
   the result is stored in the size
   
   @param a is the structure of the %vector, which size is requested
   @param size is real number, where result is stored
   
   @return always zero
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long sizev(const vector &a, double &size)
{
  size = 0.0;
  for (long i=0; i < a.n; i++)
    size += a.a[i] * a.a[i];
  size = sqrt(size);
  
  return(0);
}

double sizev (const double *a,long n)
{
  long i;
  double s;
  
  s=0.0;
  for (i=0;i<n;i++){
    s+=a[i]*a[i];
  }
  return sqrt(s);
}

/**
   The function produces size of the %ivector given by a,
   the result is stored in the size
   
   @param a is the structure of the ivector, which size is requested
   @param size is real number, where result is stored
   
   @retval 0 always
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long sizev(const ivector &a, double &size)
{
  size = 0.0;
  for (long i=0; i < a.n; i++)
    size += a.a[i] * a.a[i];
  size = sqrt(size);
  
  return(0);
}


/**
   The function produces cosine of the angle between %vector given by a and b
   the result is stored in the cos
   
   @param a is the structure of the first %vector
   @param b is the structure of the second %vector
   @param cos is real number, type double, where result is stored
   
   @b Requests :
   vectors a nd b should have the same dimension
   
   @retval 0 on succes
   @retval 1 in case
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cosav(const vector &a, const vector &b, double &cos)
{
  if (a.n != b.n)
  {
    fprintf(stderr,"\n\nError solving cos angle of vectors - incompatible size\n");
    fprintf(stderr," function cosav() (vector.cpp)\n");
    return (1);
  }
  
  double scalar, size_a, size_b;
  scprd(a, b, scalar);
  sizev(a, size_a);
  sizev(b, size_b);
  cos = scalar/(size_a * size_b);
  
  return (0);
}

/**
   The function produces cosine of the angle between %ivector given by a and b
   the result is stored in the cos
   
   @param a is the structure of the first %ivector
   @param b is the structure of the second %ivector
   @param cos is real number, type double, where result is stored
   
   @b Requests :
   vectors a nd b should have the same dimension

   @retval 0 on succes
   @retval 1 in case
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long cosav(const ivector &a, const ivector &b, double &cos)
{
  if (a.n != b.n)
  {
    fprintf(stderr,"\n\nError solving cos angle of ivectors - incompatible size\n");
    fprintf(stderr," function cosav() (ivector.cpp)\n");
    return (1);
  }
  
  long scalar;
  double size_a, size_b;
  scprd(a, b, scalar);
  sizev(a, size_a);
  sizev(b, size_b);
  cos = scalar/(size_a * size_b);
  
  return (0);
}



/**
   The function changes components of %vector a to their absolute value
   
   @param a is the structure of the first %ivector
   
   created  7.2008 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void absv(vector &a)
{
  long i;
  for (i=0; i<a.n; i++)
    a[i] = fabs(a[i]);
}



/**
   The function changes components of %vector a to their absolute value
   
   @param a is the structure of the first %ivector
   
   created  7.2008 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
void absv(ivector &a)
{
  long i;
  for (i=0; i<a.n; i++)
    a[i] = labs(a[i]);
}



/**
   The function prints out the contents of the %vector u to the stdout file,
   in precision 3 digits, width of the number is 11
   optionally, can be specified file to print, precision and the field width
   
   @param u is the structure of the %vector which is printed
   
   @b Optionally :
   @param out   is the structure with opened file for %vector output
   @param prec  is desired precision of the %vector elements on the output
   @param width is desired width of the %vector elements on the output
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const vector &u, FILE *out = stdout, int prec = 3, int width = 11)
{
  if (out == NULL)
  {
    fprintf(stderr,"\n\nError printing vector - parameter for output is NULL pointer\n");
    fprintf(stderr," function printv() (vector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < u.n; i++)
    fprintf(out, "%*.*e\n", width, prec, u.a[i]);
  return (0);
}

/**
   The function prints out the contents of the %ivector u to the stdout file,
   width of the number is 11
   optionally, can be specified file to print and the field width
   
   @param u is the structure of the %ivector which is printed
   
   @b Optionally :
   @param out   is the structure with opened file for %ivector output
   @param width is desired width of the %ivector elements on the output
   
   @retval 0 on succes
   @retval 1 in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const ivector &u, FILE *out = stdout, int width = 11)
{
  if (out == NULL)
  {
    fprintf(stderr,"\n\nError printing ivector - parameter for output is NULL pointer\n");
    fprintf(stderr," function printv() (ivector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < u.n; i++)
    fprintf(out, "%*ld\n", width, u.a[i]);
  return (0);
}


/**
   The function prints out the contents of the %vector u to the stdout file,
   in precision 3 digits, width of the number is 11
   optionally, can be specified file to print, precision and the field width
   
   @param u - pointer to array of double values which will be printed
   @param n - size of array u
   
   @b Optionally :
   @param out   is the structure with opened file for %vector output
   @param prec  is desired precision of the %vector elements on the output
   @param width is desired width of the %vector elements on the output
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const double *u, long n, FILE *out = stdout, int prec = 3, int width = 11)
{
  if (out == NULL)
  {
    print_err("parameter for output is NULL pointer", __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < n; i++)
    fprintf(out, "%*.*e\n", width, prec, u[i]);
  return (0);
}

/**
   The function prints out the contents of the %vector u (integer values are assumed) to the stdout file,
   width of the number is 11
   optionally, can be specified file to print and the field width
   
   @param u - pointer to array of integer values which will be printed
   @param n - size of array u
   
   @b Optionally :
   @param out   is the structure with opened file for %ivector output
   @param width is desired width of the %ivector elements on the output
   
   @retval 0 on succes
   @retval 1 in case NULL pointer of the out parameter
   
   created  29.5.2000 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long printv(const long *u, long n, FILE *out = stdout, int width = 11)
{
  if (out == NULL)
  {
    print_err("parameter for output is NULL pointer", __FILE__, __LINE__, __func__);
    return (1);
  }
  for (long i = 0; i < n; i++)
    fprintf(out, "%*ld\n", width, u[i]);
  return (0);
}


/**
   The function reads contents of the %vector u from the file in,
   
   @param in    pointer to the structure with opened input file
   @param u     vector which contents will be read
   
   @b Requirements :
   Vector u have to be allocated memory.
   Number of read values is given by the %vector n member.
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the in parameter
   @retval 2 : in case error reading number from the file
   
   created  6.9.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long readv(FILE *in, vector &u)
{
  if (in == NULL)
  {
    fprintf(stderr,"\n\nError reading vector - parameter for input file is NULL pointer\n");
    fprintf(stderr," function readv() (vector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < u.n; i++)
  {
    if (fscanf(in, "%le", &u.a[i]) != 1)
      return(2);
  }
  return (0);
}

/**
   The function reads contents of the %vector u from the file in,
   
   @param in    pointer to the structure with opened input file
   @param u     vector which contents will be read
   
   @b Requirements :
   Vector u have to be allocated memory.
   Number of read values is given by the %vector n member.
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the in parameter
   @retval 2 : in case error reading number from the file
   
   created  3.12.2006 by Jaroslav Kruis, jk@cml.fsv.cvut.cz
*/
long readv(XFILE *in, vector &u)
{
  if (in == NULL)
  {
    fprintf(stderr,"\n\nError reading vector - parameter for input file is NULL pointer\n");
    fprintf(stderr," function readv() (vector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < u.n; i++)
  {
    if (xfscanf(in, "%le", &u.a[i]) != 1)
      return(2);
  }
  return (0);
}

/**
   The function reads contents of the %vector u from the string in,

   @param in    pointer to the input string
   @param u     vector which contents will be read
   
   @b Requirements :
   Vector u have to be allocated memory.
   Number of read values is given by the vector n member.
   
   @retval 0 : on succes
   @retval 1 : in case NULL pointer of the in parameter
   @retval 2 : in case error reading number from the file
   
   created  6.9.2001 by Tomas Koudelka, koudelka@cml.fsv.cvut.cz
*/
long readv(char *in, vector &u)
{
  if (in == NULL)
  {
    fprintf(stderr,"\n\nError reading vector - parameter for input string is NULL pointer\n");
    fprintf(stderr," function readv() (vector.cpp)\n");
    return (1);
  }
  for (long i = 0; i < u.n; i++)
  {
    if (sscanf(in, "%le", &u.a[i]) != 1)
      return(2);
  }
  return (0);
}


/**
   function extracts ncomp components from %vector b
   and put them into %vector a

   @param a - %vector containing extracted components
   @param b - %vector from where components are extracted
   @param fi - first index
   @param ncomp - number of components

   30.11.2002
*/
void extract (vector &a,vector &b,long fi, long ncomp)
{
  long i,j;

  j=fi;
  for (i=0;i<ncomp;i++){
    a[i]=b[j];
    j++;
  }
}

/**
   Function extracts positive components from vector a
   and puts them into vector b in corresponding position as in the vector a.
   If component is negative the zero is stored.

   @param a - %vector containing compared components
   @param b - %vector where extracted components are stored

   3.3.2003
*/
long extractposv (vector &a,vector &b)
{
  long i;
  if (a.n != b.n)
  {
    fprintf(stderr,"\n\nError extracting positive numbers - incompatible size\n");
    fprintf(stderr," in function extractposv, (%s, line %d)\n", __FILE__, __LINE__);
    return (1);
  }
  for (i = 0; i < a.n; i++)
  {
    if (a[i] > 0.0)
      b[i] = a[i];
    else
      b[i] = 0.0;
  }
  return 0;
}

/**
   Function extracts negative components from vector a
   and puts them into vector b in corresponding position as in the vector a.
   If component is positive the zero is stored.

   @param a - %vector containing compared components
   @param b - %vector where extracted components are stored

   25.2.2004
*/
long extractnegv (vector &a,vector &b)
{
  long i;
  if (a.n != b.n)
  {
    fprintf(stderr,"\n\nError extracting negative numbers - incompatible size\n");
    fprintf(stderr," in function extractnegv, (%s, line %d)\n", __FILE__, __LINE__);
    return (1);
  }
  for (i = 0; i < a.n; i++)
  {
    if (a[i] < 0.0)
      b[i] = a[i];
    else
      b[i] = 0.0;
  }
  return 0;
}

/**
   function reads number of attributes and order of attributes used in stochastic or fuzzy computations
   
   @param in - pointer to input file
   
   JK, 23.8.2005
*/
long atsel::read(XFILE *in)
{
  long i;
  
  //  number of all attributes
  //  number of basic attributes
  //  number of hidden attributes
  //
  //  number of basic attributes should be usually equal to the number of all attributes
  //  it differs in case of plastic material, where parameters of the model are stored first (nba components)
  //  and hardening/softening parameters are stored behind them (nia components)
  xfscanf(in, "%ld %ld %ld", &num,&nba,&nia);
  
  if (nba+nia != num){
    fprintf (stderr,"\n\n wrong number of all attributes, basic attributes and hidden attributes (file %s, line %d),\n",__FILE__,__LINE__);
  }
  
  atrib = new long[num];
  for (i = 0; i < num; i++){
    xfscanf(in, "%ld", atrib+i);
    atrib[i]--;
  }
  return 0;
}
