#ifndef DEF_VECTOR
#define DEF_VECTOR
#include <stdio.h>
#include <assert.h>
//#define DEBUG_VECTOR

#include "iotools.h"

#ifdef DEBUG_VECTOR
 static unsigned long Acv; ///< allocation counter
#endif 

struct vector
/**
   This file declares struct of the vector, which implements
   %vector with %elemnts type of double.
   There is also declarations of the functions for the %vector computing.
*/
{
  long n;    ///< number of %vector elements
  double *a; ///< pointer to memory ,where are stored elements of %vector
  
  vector() { n = 0; a = NULL;}; ///< default constructor
  vector(long n);               ///< allocating constructor
  vector(const vector &v);      ///< copy constructor
  
  /**
     The function operator enables access to the elements of the member array a
     
     @param i is the number of the desired %vector element
     @retval Returns i-th element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double& operator () (long i)
  {
    #ifdef DEBUG_VECTOR
     assert(i < n);
    #endif
    return (a[i]);
  };
  
  /**
     The array index operator enables access to the elements of the member array a
     
     @param i is the number of the desired %vector %element
     @retval i-th %element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double& operator [] (long i)
  {
    #ifdef DEBUG_VECTOR
     assert(i < n);
    #endif
    return (a[i]);
  };
  
  /**
     The array index operator enables CONSTANT access to the elements of the member array a
     
     @param i is the number of the desired %vector element
     @retval i-th %element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  double operator [] (long i) const
  {
    #ifdef DEBUG_VECTOR
     assert(i < n);
    #endif
    return (a[i]);
  };
  
  ~vector(); ///< destructor
};


struct ivector
/**
   This file declares struct of the vector, which implements
   %vector with %elemnts type of long.
*/
{
  long n;   ///< number of %vector elements
  long *a;  ///< pointer to memory ,where are stored elements of %vector
  
  ivector() { n = 0; a = NULL;}; ///< default constructor
  ivector(long n);               ///< allocating constructor
  ivector(const ivector &v);     ///< copy constructor
  
  /**
     The function operator enables access to the elements of the member array a
     
     @param i is the number of the desired %vector element
     @retval Returns i-th element of %vector
     
     created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
     modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long& operator () (long i)
  {
    #ifdef DEBUG_VECTOR
     assert(i < n);
    #endif
    return (a[i]);
  };

  /**
   The array index operator enables access to the elements of the member array a
   
   @param i is the number of the desired %vector %element
   @retval i-th %element of %vector
   
   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long& operator [] (long i)
  {
    #ifdef DEBUG_VECTOR
     assert(i < n);
    #endif
    return (a[i]);
  };

  /**
   The array index operator enables CONSTANT access to the elements of the member array a
   
   @param i is the number of the desired %vector %element
   @retval i-th %element of %vector
   
   created  29.5.2000, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
   modified  9.5.2001, Tomas Koudelka, koudelka@cml.fsv.cvut.cz
  */
  long& operator [] (long i) const
  {
    #ifdef DEBUG_VECTOR
     assert(i < n);
    #endif
    return (a[i]);
  };

  ~ivector(); ///< destructor
};

/**
   this structure serves for defined attributes
   it is used e.g. in stochastic analysis
   
   TKo
*/
struct atsel{
  ///  number of components
  long num;
  ///  number of basic components (optional parameter)
  long nba;
  ///  number of internal (hidden) components (optional parameter)
  long nia;

  long *atrib;

  atsel (){
    num=0;  
    nba=0;  
    nia=0;  
    atrib=NULL;
  };
  ~atsel (){  delete [] atrib;  };

  long read(XFILE* in);
};


/// allocates %vector
long allocv(long n, vector &vec);
/// allocates %ivector
long allocv(long n, ivector &vec);

/// copies contents of %vector
long copyv(const vector &src, vector &dest);
/// copies contents of %ivector
long copyv(const ivector &src, ivector &dest);
long copyv(const long *src,long *dest,long n);
/// copies contents of array
long copyv(const double *src, double *dest, long n);
/// copies contents of array to vector
long copyv(const double *src, vector &dest);
/// copies contents of vector to array
long copyv(const vector &src, double *dest);



/// fills contents of %vector with given value
long fillv(double c, vector &vec);
/// fills contents of %ivector with given value
long fillv(long c, ivector &vec);
void fillv(long c, long *vec,long n);
void fillv(double c, double *vec,long n);

/// fills array with zeros
long nullv (double *a,long n);
long nullv (ivector &a);

/// deallocates %vector
long destrv(vector &vec);
/// deallocates %ivector
long destrv(ivector &vec);

/// adds 2 vectors
long addv(const vector &a, const vector &b, vector &c);
/// adds 2 ivectors
long addv(const ivector &a, const ivector &b, ivector &c);

/// adds 2 double arrays
void addv(double *a,double *b,long n);

/// adds 2 double arrays
void addv(double *a,double *b, double *c, long n);

/// adds 2 double arrays, 2nd array is multplied by constant
void addmultv (double *a,double *b,double c,long n);

/// substracts 2 vectors
long subv(const vector &a, const vector &b, vector &c);
/// substracts 2 ivectors
long subv(const ivector &a, const ivector &b, ivector &c);

/// substracts 2 arrays
void subv(double *a,double *b,long n);
/// substracts 2 arrays, result is stored to the another array
void subv (double *a,double *b,double *c,long n);

/// performs cross product of 2 vectors
long crprd(const vector &a, const vector &b, vector &c);
/// performs cross product of 2 ivectors
long crprd(const ivector &a, const ivector &b, ivector &c);

/// performs scalar product of 2 vectors
long scprd(const vector &a, const vector &b, double &scalar);
/// performs scalar product of 2 ivectors
long scprd(const ivector &a, const ivector &b, long &scalar);
/// performs scalar product of 2 arrays
double ss (double *a,double *b,long n);

/// multiplies %vector by constant
long cmulv(double c, const vector &a, vector &u);
/// multiplies %ivector by constant of type double
long cmulv(double c, const ivector &a, vector &u);
/// multiplies %ivector by constant of type long
long cmulv(long c, const ivector &a, ivector &u);

/// multiplies %vector by constant
long cmulv(double c, vector &a);
/// multiplies %ivector by constant of type long
long cmulv(long c, ivector &a);
/// multiplies double array by constant
void cmulv(double c, double *a, long n);

/// produces size of %vector
long sizev(const vector &a, double &size);
double sizev(const double *a, long n);
/// produces size of %ivector
long sizev(const ivector &a, double &size);

/// cosine angle of 2 vectors
long cosav(const vector &a, const vector &b, double &cos);
/// cosine angle of 2 ivectors
long cosav(const ivector &a, const ivector &b, double &cos);

/// absolute value of vector components
void absv(vector &a);
/// absolute value of vector components
void absv(ivector &a);

/// prints contents of %vector
long printv(const vector &u, FILE *out, int prec, int width);
/// prints contents of %ivector
long printv(const ivector &u, FILE *out, int prec, int width);
/// prints contents of %vector
long printv(const double *u, long n, FILE *out, int prec, int width);
/// prints contents of %ivector
long printv(const long *u, long n, FILE *out, int prec, int width);

/// reads contents of %vector from file
long readv(FILE *in, vector &u);
long readv(XFILE *in, vector &u);

/// reads contents of %vector from string
long readv(char *in, vector &u);

/// function extracts ncomp components (from index fi) from %vector b
void extract (vector &a, vector &b, long fi, long ncomp);

/// extracts only positive components from %vector a and stores them to %vector b
long extractposv (vector &a, vector &b);
/// extracts only negative components from %vector a and stores them to %vector b
long extractnegv (vector &a, vector &b);


#endif
